@@ -17,8 +17,8 @@ namespace boost { namespace fusion
     }
 
     template<typename Seq, typename State, typename F>
-    BOOST_FUSION_GPU_ENABLED
-    typename result_of::fold<
+    BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+    inline typename result_of::fold<
         Seq
       , State const
       , F
@@ -26,8 +26,8 @@ namespace boost { namespace fusion
     fold(Seq& seq, State const& state, F f);
 
     template<typename Seq, typename State, typename F>
-    BOOST_FUSION_GPU_ENABLED
-    typename result_of::fold<
+    BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+    inline typename result_of::fold<
         Seq const
       , State const
       , F
@@ -35,19 +35,19 @@ namespace boost { namespace fusion
     fold(Seq const& seq, State const& state, F f);
 
     template<typename Seq, typename State, typename F>
-    BOOST_FUSION_GPU_ENABLED
-    typename result_of::fold<
+    BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+    inline typename result_of::fold<
         Seq
-      , State const
+      , State
       , F
     >::type
     fold(Seq& seq, State& state, F f);
 
     template<typename Seq, typename State, typename F>
-    BOOST_FUSION_GPU_ENABLED
-    typename result_of::fold<
+    BOOST_CONSTEXPR BOOST_FUSION_GPU_ENABLED
+    inline typename result_of::fold<
         Seq const
-      , State const
+      , State
       , F
     >::type
     fold(Seq const& seq, State& state, F f);
