@@ -3,7 +3,12 @@
 // Copyright (c) 2007-2013 Barend Gehrels, Amsterdam, the Netherlands.
 // Copyright (c) 2008-2013 Bruno Lalande, Paris, France.
 // Copyright (c) 2009-2013 Mateusz Loskot, London, UK.
-// Copyright (c) 2013-2014 Adam Wulkiewicz, Lodz, Poland.
+// Copyright (c) 2013-2017 Adam Wulkiewicz, Lodz, Poland.
+
+// This file was modified by Oracle on 2017-2021.
+// Modifications copyright (c) 2017-2021 Oracle and/or its affiliates.
+
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Use, modification and distribution is subject to the Boost Software License,
 // Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
@@ -15,11 +20,17 @@
 
 #include <cstddef>
 
-#include <boost/range.hpp>
+#include <boost/range/begin.hpp>
+#include <boost/range/end.hpp>
+#include <boost/range/size.hpp>
+#include <boost/range/value_type.hpp>
 
-#include <boost/geometry/algorithms/detail/interior_iterator.hpp>
+#include <boost/geometry/algorithms/detail/assign_box_corners.hpp>
+
+#include <boost/geometry/arithmetic/arithmetic.hpp>
 
 #include <boost/geometry/core/cs.hpp>
+#include <boost/geometry/core/point_order.hpp>
 #include <boost/geometry/core/point_type.hpp>
 #include <boost/geometry/core/ring_type.hpp>
 #include <boost/geometry/core/tags.hpp>
@@ -27,8 +38,6 @@
 #include <boost/geometry/geometries/concepts/check.hpp>
 #include <boost/geometry/iterators/ever_circling_iterator.hpp>
 
-#include <boost/geometry/algorithms/detail/assign_box_corners.hpp>
-
 #include <boost/geometry/strategies/side.hpp>
 
 #include <boost/geometry/util/math.hpp>
@@ -89,7 +98,8 @@ inline void move_along_vector(PointType& point, PointType const& extreme, Coordi
 
     CoordinateType const base_diff = base_value - geometry::get<Dimension>(extreme);
 
-    multiply_value(vector, base_diff / diff);
+    multiply_value(vector, base_diff);
+    divide_value(vector, diff);
 
     // The real move:
     point = extreme;
@@ -112,15 +122,8 @@ template<typename Ring, std::size_t Dimension>
 struct extreme_points_on_ring
 {
 
-    typedef typename geometry::coordinate_type<Ring>::type coordinate_type;
-    typedef typename boost::range_iterator<Ring const>::type range_iterator;
-    typedef typename geometry::point_type<Ring>::type point_type;
-
-    typedef typename geometry::strategy::side::services::default_strategy
-        <
-            typename geometry::cs_tag<point_type>::type
-        >::type side_strategy;
-
+    using coordinate_type = geometry::coordinate_type_t<Ring>;
+    using point_type = geometry::point_type_t<Ring>;
 
     template <typename CirclingIterator, typename Points>
     static inline bool extend(CirclingIterator& it,
@@ -213,10 +216,11 @@ struct extreme_points_on_ring
         return true;
     }
 
-    template <typename Extremes, typename Intruders, typename CirclingIterator>
+    template <typename Extremes, typename Intruders, typename CirclingIterator, typename SideStrategy>
     static inline void get_intruders(Ring const& ring, CirclingIterator left, CirclingIterator right,
             Extremes const& extremes,
-            Intruders& intruders)
+            Intruders& intruders,
+            SideStrategy const& strategy)
     {
         if (boost::size(extremes) < 3)
         {
@@ -236,8 +240,9 @@ struct extreme_points_on_ring
             coordinate_type const other_coordinate = geometry::get<1 - Dimension>(*right);
             if (coordinate > min_value && other_coordinate > other_min && other_coordinate < other_max)
             {
-                int const first_side = side_strategy::apply(*right, extremes.front(), *(extremes.begin() + 1));
-                int const last_side = side_strategy::apply(*right, *(extremes.rbegin() + 1), extremes.back());
+                int const factor = geometry::point_order<Ring>::value == geometry::clockwise ? 1 : -1;
+                int const first_side = strategy.apply(*right, extremes.front(), *(extremes.begin() + 1)) * factor;
+                int const last_side = strategy.apply(*right, *(extremes.rbegin() + 1), extremes.back()) * factor;
 
                 // If not lying left from any of the extemes side
                 if (first_side != 1 && last_side != 1)
@@ -261,10 +266,11 @@ struct extreme_points_on_ring
         }
     }
 
-    template <typename Extremes, typename Intruders>
+    template <typename Extremes, typename Intruders, typename SideStrategy>
     static inline void get_intruders(Ring const& ring,
             Extremes const& extremes,
-            Intruders& intruders)
+            Intruders& intruders,
+            SideStrategy const& strategy)
     {
         std::size_t const n = boost::size(ring);
         if (n >= 3)
@@ -273,14 +279,14 @@ struct extreme_points_on_ring
             geometry::ever_circling_range_iterator<Ring const> right(ring);
             ++right;
 
-            get_intruders(ring, left, right, extremes, intruders);
+            get_intruders(ring, left, right, extremes, intruders, strategy);
         }
     }
 
-    template <typename Iterator>
-    static inline bool right_turn(Ring const& ring, Iterator it)
+    template <typename Iterator, typename SideStrategy>
+    static inline bool right_turn(Ring const& ring, Iterator it, SideStrategy const& strategy)
     {
-        int const index = std::distance(boost::begin(ring), it);
+        auto const index = std::distance(boost::begin(ring), it);
         geometry::ever_circling_range_iterator<Ring const> left(ring);
         geometry::ever_circling_range_iterator<Ring const> right(ring);
         left += index;
@@ -291,8 +297,9 @@ struct extreme_points_on_ring
             return false;
         }
 
-        int const first_side = side_strategy::apply(*(right - 1), *right, *left);
-        int const last_side = side_strategy::apply(*left, *(left + 1), *right);
+        int const factor = geometry::point_order<Ring>::value == geometry::clockwise ? 1 : -1;
+        int const first_side = strategy.apply(*(right - 1), *right, *left) * factor;
+        int const last_side = strategy.apply(*left, *(left + 1), *right) * factor;
 
 //std::cout << "Candidate at " << geometry::wkt(*it) << " first=" << first_side << " last=" << last_side << std::endl;
 
@@ -302,8 +309,11 @@ struct extreme_points_on_ring
 
 
     // Gets the extreme segments (top point plus neighbouring points), plus intruders, if any, on the same ring
-    template <typename Extremes, typename Intruders>
-    static inline bool apply(Ring const& ring, Extremes& extremes, Intruders& intruders)
+    template <typename Extremes, typename Intruders, typename SideStrategy>
+    static inline bool apply(Ring const& ring,
+                             Extremes& extremes,
+                             Intruders& intruders,
+                             SideStrategy const& strategy)
     {
         std::size_t const n = boost::size(ring);
         if (n < 3)
@@ -313,11 +323,11 @@ struct extreme_points_on_ring
 
         // Get all maxima, usually one. In case of self-tangencies, or self-crossings,
         // the max might be is not valid. A valid max should make a right turn
-        range_iterator max_it = boost::begin(ring);
+        auto max_it = boost::begin(ring);
         compare<Dimension> smaller;
-        for (range_iterator it = max_it + 1; it != boost::end(ring); ++it)
+        for (auto it = max_it + 1; it != boost::end(ring); ++it)
         {
-            if (smaller(*max_it, *it) && right_turn(ring, it))
+            if (smaller(*max_it, *it) && right_turn(ring, it, strategy))
             {
                 max_it = it;
             }
@@ -328,7 +338,7 @@ struct extreme_points_on_ring
             return false;
         }
 
-        int const index = std::distance(boost::begin(ring), max_it);
+        auto const index = std::distance(boost::begin(ring), max_it);
 //std::cout << "Extreme point lies at " << index << " having " << geometry::wkt(*max_it) << std::endl;
 
         geometry::ever_circling_range_iterator<Ring const> left(ring);
@@ -360,7 +370,7 @@ struct extreme_points_on_ring
 
         std::copy(points.begin(), points.end(), std::back_inserter(extremes));
 
-        get_intruders(ring, left, right, extremes, intruders);
+        get_intruders(ring, left, right, extremes, intruders, strategy);
 
         return true;
     }
@@ -383,7 +393,7 @@ template
 <
     typename Geometry,
     std::size_t Dimension,
-    typename GeometryTag = typename tag<Geometry>::type
+    typename GeometryTag = tag_t<Geometry>
 >
 struct extreme_points
 {};
@@ -398,27 +408,26 @@ struct extreme_points<Ring, Dimension, ring_tag>
 template<typename Polygon, std::size_t Dimension>
 struct extreme_points<Polygon, Dimension, polygon_tag>
 {
-    template <typename Extremes, typename Intruders>
-    static inline bool apply(Polygon const& polygon, Extremes& extremes, Intruders& intruders)
+    template <typename Extremes, typename Intruders, typename SideStrategy>
+    static inline bool apply(Polygon const& polygon, Extremes& extremes, Intruders& intruders,
+                             SideStrategy const& strategy)
     {
-        typedef typename geometry::ring_type<Polygon>::type ring_type;
-        typedef detail::extreme_points::extreme_points_on_ring
+        using ring_implementation = detail::extreme_points::extreme_points_on_ring
             <
-                ring_type, Dimension
-            > ring_implementation;
+                geometry::ring_type_t<Polygon>, Dimension
+            >;
 
-        if (! ring_implementation::apply(geometry::exterior_ring(polygon), extremes, intruders))
+        if (! ring_implementation::apply(geometry::exterior_ring(polygon),
+                                         extremes, intruders, strategy))
         {
             return false;
         }
 
         // For a polygon, its interior rings can contain intruders
-        typename interior_return_type<Polygon const>::type
-            rings = interior_rings(polygon);
-        for (typename detail::interior_iterator<Polygon const>::type
-                it = boost::begin(rings); it != boost::end(rings); ++it)
+        auto const& rings = interior_rings(polygon);
+        for (auto it = boost::begin(rings); it != boost::end(rings); ++it)
         {
-            ring_implementation::get_intruders(*it, extremes,  intruders);
+            ring_implementation::get_intruders(*it, extremes,  intruders, strategy);
         }
 
         return true;
@@ -428,8 +437,9 @@ struct extreme_points<Polygon, Dimension, polygon_tag>
 template<typename Box>
 struct extreme_points<Box, 1, box_tag>
 {
-    template <typename Extremes, typename Intruders>
-    static inline bool apply(Box const& box, Extremes& extremes, Intruders& )
+    template <typename Extremes, typename Intruders, typename SideStrategy>
+    static inline bool apply(Box const& box, Extremes& extremes, Intruders& ,
+                             SideStrategy const& )
     {
         extremes.resize(4);
         geometry::detail::assign_box_corners_oriented<false>(box, extremes);
@@ -441,8 +451,9 @@ struct extreme_points<Box, 1, box_tag>
 template<typename Box>
 struct extreme_points<Box, 0, box_tag>
 {
-    template <typename Extremes, typename Intruders>
-    static inline bool apply(Box const& box, Extremes& extremes, Intruders& )
+    template <typename Extremes, typename Intruders, typename SideStrategy>
+    static inline bool apply(Box const& box, Extremes& extremes, Intruders& ,
+                             SideStrategy const& )
     {
         extremes.resize(4);
         geometry::detail::assign_box_corners_oriented<false>(box, extremes);
@@ -455,8 +466,9 @@ struct extreme_points<Box, 0, box_tag>
 template<typename MultiPolygon, std::size_t Dimension>
 struct extreme_points<MultiPolygon, Dimension, multi_polygon_tag>
 {
-    template <typename Extremes, typename Intruders>
-    static inline bool apply(MultiPolygon const& multi, Extremes& extremes, Intruders& intruders)
+    template <typename Extremes, typename Intruders, typename SideStrategy>
+    static inline bool apply(MultiPolygon const& multi, Extremes& extremes,
+                             Intruders& intruders, SideStrategy const& strategy)
     {
         // Get one for the very first polygon, that is (for the moment) enough.
         // It is not guaranteed the "extreme" then, but for the current purpose
@@ -468,7 +480,7 @@ struct extreme_points<MultiPolygon, Dimension, multi_polygon_tag>
                     typename boost::range_value<MultiPolygon const>::type,
                     Dimension,
                     polygon_tag
-                >::apply(*boost::begin(multi), extremes, intruders);
+                >::apply(*boost::begin(multi), extremes, intruders, strategy);
         }
 
         return false;
@@ -484,18 +496,28 @@ struct extreme_points<MultiPolygon, Dimension, multi_polygon_tag>
        for Edge=0 in dimension 0, the right side)
 \note We could specify a strategy (less/greater) to get bottom/left side too. However, until now we don't need that.
  */
-template <std::size_t Edge, typename Geometry, typename Extremes, typename Intruders>
-inline bool extreme_points(Geometry const& geometry, Extremes& extremes, Intruders& intruders)
+template
+<
+    std::size_t Edge,
+    typename Geometry,
+    typename Extremes,
+    typename Intruders,
+    typename SideStrategy
+>
+inline bool extreme_points(Geometry const& geometry,
+                           Extremes& extremes,
+                           Intruders& intruders,
+                           SideStrategy const& strategy)
 {
-    concept::check<Geometry const>();
+    concepts::check<Geometry const>();
 
     // Extremes is not required to follow a geometry concept (but it should support an output iterator),
     // but its elements should fulfil the point-concept
-    concept::check<typename boost::range_value<Extremes>::type>();
+    concepts::check<typename boost::range_value<Extremes>::type>();
 
     // Intruders should contain collections which value type is point-concept
     // Extremes might be anything (supporting an output iterator), but its elements should fulfil the point-concept
-    concept::check
+    concepts::check
         <
             typename boost::range_value
                 <
@@ -504,10 +526,32 @@ inline bool extreme_points(Geometry const& geometry, Extremes& extremes, Intrude
             const
         >();
 
-    return dispatch::extreme_points<Geometry, Edge>::apply(geometry, extremes, intruders);
+    return dispatch::extreme_points
+            <
+                Geometry,
+                Edge
+            >::apply(geometry, extremes, intruders, strategy);
 }
 
 
+template
+<
+    std::size_t Edge,
+    typename Geometry,
+    typename Extremes,
+    typename Intruders
+>
+inline bool extreme_points(Geometry const& geometry,
+                           Extremes& extremes,
+                           Intruders& intruders)
+{
+    using strategy_type = typename strategy::side::services::default_strategy
+            <
+                cs_tag_t<Geometry>
+            >::type;
+
+    return geometry::extreme_points<Edge>(geometry,extremes, intruders, strategy_type());
+}
 
 }} // namespace boost::geometry
 
