@@ -4,6 +4,10 @@
 // Copyright (c) 2008-2012 Bruno Lalande, Paris, France.
 // Copyright (c) 2009-2012 Mateusz Loskot, London, UK.
 
+// This file was modified by Oracle on 2020.
+// Modifications copyright (c) 2020 Oracle and/or its affiliates.
+// Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
+
 // Parts of Boost.Geometry are redesigned from Geodan's Geographic Library
 // (geolib/GGL), copyright (c) 1995-2010 Geodan, Amsterdam, the Netherlands.
 
@@ -14,7 +18,8 @@
 #ifndef BOOST_GEOMETRY_ALGORITHMS_DETAIL_MULTI_SUM_HPP
 #define BOOST_GEOMETRY_ALGORITHMS_DETAIL_MULTI_SUM_HPP
 
-#include <boost/range.hpp>
+#include <boost/range/begin.hpp>
+#include <boost/range/end.hpp>
 
 
 namespace boost { namespace geometry
@@ -26,15 +31,10 @@ namespace detail
 struct multi_sum
 {
     template <typename ReturnType, typename Policy, typename MultiGeometry, typename Strategy>
-    static inline ReturnType apply(MultiGeometry const& geometry, Strategy const& strategy)
+    static inline ReturnType apply(MultiGeometry const& multi, Strategy const& strategy)
     {
         ReturnType sum = ReturnType();
-        for (typename boost::range_iterator
-                <
-                    MultiGeometry const
-                >::type it = boost::begin(geometry);
-            it != boost::end(geometry);
-            ++it)
+        for (auto it = boost::begin(multi); it != boost::end(multi); ++it)
         {
             sum += Policy::apply(*it, strategy);
         }
