@@ -3,8 +3,9 @@
 // Copyright (c) 2007-2014 Barend Gehrels, Amsterdam, the Netherlands.
 // Copyright (c) 2017 Adam Wulkiewicz, Lodz, Poland.
 
-// This file was modified by Oracle on 2014, 2017.
-// Modifications copyright (c) 2014-2017 Oracle and/or its affiliates.
+// This file was modified by Oracle on 2014-2024.
+// Modifications copyright (c) 2014-2024 Oracle and/or its affiliates.
+// Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Menelaos Karavelas, on behalf of Oracle
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
@@ -16,19 +17,24 @@
 #define BOOST_GEOMETRY_ALGORITHMS_DETAIL_OVERLAY_FOLLOW_HPP
 
 #include <cstddef>
+#include <type_traits>
 
-#include <boost/range.hpp>
-#include <boost/mpl/assert.hpp>
+#include <boost/range/begin.hpp>
+#include <boost/range/end.hpp>
+#include <boost/range/size.hpp>
+#include <boost/range/value_type.hpp>
 
-#include <boost/geometry/algorithms/detail/point_on_border.hpp>
+#include <boost/geometry/algorithms/clear.hpp>
+#include <boost/geometry/algorithms/detail/covered_by/implementation.hpp>
 #include <boost/geometry/algorithms/detail/overlay/append_no_duplicates.hpp>
 #include <boost/geometry/algorithms/detail/overlay/copy_segments.hpp>
+#include <boost/geometry/algorithms/detail/overlay/debug_traverse.hpp>
 #include <boost/geometry/algorithms/detail/overlay/turn_info.hpp>
-
-#include <boost/geometry/algorithms/covered_by.hpp>
-#include <boost/geometry/algorithms/clear.hpp>
+#include <boost/geometry/algorithms/detail/point_on_border.hpp>
 #include <boost/geometry/algorithms/detail/relate/turns.hpp>
-
+#include <boost/geometry/algorithms/detail/tupled_output.hpp>
+#include <boost/geometry/core/static_assert.hpp>
+#include <boost/geometry/util/condition.hpp>
 
 namespace boost { namespace geometry
 {
@@ -42,7 +48,7 @@ namespace following
 {
 
 template <typename Turn, typename Operation>
-static inline bool is_entering(Turn const& /* TODO remove this parameter */, Operation const& op)
+inline bool is_entering(Turn const& /* TODO remove this parameter */, Operation const& op)
 {
     // (Blocked means: blocked for polygon/polygon intersection, because
     // they are reversed. But for polygon/line it is similar to continue)
@@ -58,11 +64,11 @@ template
     typename Operation,
     typename LineString,
     typename Polygon,
-    typename PtInPolyStrategy
+    typename Strategy
 >
-static inline bool last_covered_by(Turn const& /*turn*/, Operation const& op,
+inline bool last_covered_by(Turn const& /*turn*/, Operation const& op,
                 LineString const& linestring, Polygon const& polygon,
-                PtInPolyStrategy const& strategy)
+                Strategy const& strategy)
 {
     return geometry::covered_by(range::at(linestring, op.seg_id.segment_index), polygon, strategy);
 }
@@ -74,18 +80,20 @@ template
     typename Operation,
     typename LineString,
     typename Polygon,
-    typename PtInPolyStrategy
+    typename Strategy
 >
-static inline bool is_leaving(Turn const& turn, Operation const& op,
+inline bool is_leaving(Turn const& turn, Operation const& op,
                 bool entered, bool first,
                 LineString const& linestring, Polygon const& polygon,
-                PtInPolyStrategy const& strategy)
+                Strategy const& strategy)
 {
     if (op.operation == operation_union)
     {
         return entered
             || turn.method == method_crosses
-            || (first && last_covered_by(turn, op, linestring, polygon, strategy))
+            || (first
+                && op.position != position_front
+                && last_covered_by(turn, op, linestring, polygon, strategy))
             ;
     }
     return false;
@@ -98,12 +106,12 @@ template
     typename Operation,
     typename LineString,
     typename Polygon,
-    typename PtInPolyStrategy
+    typename Strategy
 >
-static inline bool is_staying_inside(Turn const& turn, Operation const& op,
+inline bool is_staying_inside(Turn const& turn, Operation const& op,
                 bool entered, bool first,
                 LineString const& linestring, Polygon const& polygon,
-                PtInPolyStrategy const& strategy)
+                Strategy const& strategy)
 {
     if (turn.method == method_crosses)
     {
@@ -126,11 +134,11 @@ template
     typename Operation,
     typename Linestring,
     typename Polygon,
-    typename PtInPolyStrategy
+    typename Strategy
 >
-static inline bool was_entered(Turn const& turn, Operation const& op, bool first,
+inline bool was_entered(Turn const& turn, Operation const& op, bool first,
                 Linestring const& linestring, Polygon const& polygon,
-                PtInPolyStrategy const& strategy)
+                Strategy const& strategy)
 {
     if (first && (turn.method == method_collinear || turn.method == method_equal))
     {
@@ -139,13 +147,68 @@ static inline bool was_entered(Turn const& turn, Operation const& op, bool first
     return false;
 }
 
+template
+<
+    typename Turn,
+    typename Operation
+>
+inline bool is_touching(Turn const& turn, Operation const& op,
+                        bool entered)
+{
+    return (op.operation == operation_union || op.operation == operation_blocked)
+        && (turn.method == method_touch || turn.method == method_touch_interior)
+        && !entered
+        && !op.is_collinear;
+}
+
+
+template
+<
+    typename GeometryOut,
+    typename Tag = geometry::tag_t<GeometryOut>
+>
+struct add_isolated_point
+{};
+
+template <typename LineStringOut>
+struct add_isolated_point<LineStringOut, linestring_tag>
+{
+    template <typename Point, typename OutputIterator>
+    static inline void apply(Point const& point, OutputIterator& out)
+    {
+        LineStringOut isolated_point_ls;
+        geometry::append(isolated_point_ls, point);
+
+#ifndef BOOST_GEOMETRY_ALLOW_ONE_POINT_LINESTRINGS
+        geometry::append(isolated_point_ls, point);
+#endif // BOOST_GEOMETRY_ALLOW_ONE_POINT_LINESTRINGS
+
+        *out++ = isolated_point_ls;
+    }
+};
+
+template <typename PointOut>
+struct add_isolated_point<PointOut, point_tag>
+{
+    template <typename Point, typename OutputIterator>
+    static inline void apply(Point const& point, OutputIterator& out)
+    {
+        PointOut isolated_point;
+
+        geometry::detail::conversion::convert_point_to_point(point, isolated_point);
+
+        *out++ = isolated_point;
+    }
+};
+
 
 // Template specialization structure to call the right actions for the right type
 template <overlay_type OverlayType, bool RemoveSpikes = true>
 struct action_selector
 {
-    // If you get here the overlay type is not intersection or difference
-    // BOOST_MPL_ASSERT(false);
+    BOOST_GEOMETRY_STATIC_ASSERT_FALSE(
+        "If you get here the overlay type is not intersection or difference.",
+        std::integral_constant<overlay_type, OverlayType>);
 };
 
 // Specialization for intersection, containing the implementation
@@ -159,21 +222,19 @@ struct action_selector<overlay_intersection, RemoveSpikes>
         typename LineString,
         typename Point,
         typename Operation,
-        typename SideStrategy,
-        typename RobustPolicy
+        typename Strategy
     >
     static inline void enter(LineStringOut& current_piece,
                 LineString const& ,
                 segment_identifier& segment_id,
                 signed_size_type , Point const& point,
                 Operation const& operation,
-                SideStrategy const& ,
-                RobustPolicy const& ,
+                Strategy const& strategy,
                 OutputIterator& )
     {
         // On enter, append the intersection point and remember starting point
         // TODO: we don't check on spikes for linestrings (?). Consider this.
-        detail::overlay::append_no_duplicates(current_piece, point);
+        detail::overlay::append_no_duplicates(current_piece, point, strategy);
         segment_id = operation.seg_id;
     }
 
@@ -184,16 +245,14 @@ struct action_selector<overlay_intersection, RemoveSpikes>
         typename LineString,
         typename Point,
         typename Operation,
-        typename SideStrategy,
-        typename RobustPolicy
+        typename Strategy
     >
     static inline void leave(LineStringOut& current_piece,
                 LineString const& linestring,
                 segment_identifier& segment_id,
                 signed_size_type index, Point const& point,
                 Operation const& ,
-                SideStrategy const& strategy,
-                RobustPolicy const& robust_policy,
+                Strategy const& strategy,
                 OutputIterator& out)
     {
         // On leave, copy all segments from starting point, append the intersection point
@@ -201,8 +260,8 @@ struct action_selector<overlay_intersection, RemoveSpikes>
         detail::copy_segments::copy_segments_linestring
             <
                 false, RemoveSpikes
-            >::apply(linestring, segment_id, index, strategy, robust_policy, current_piece);
-        detail::overlay::append_no_duplicates(current_piece, point);
+            >::apply(linestring, segment_id, index, strategy, current_piece);
+        detail::overlay::append_no_duplicates(current_piece, point, strategy);
         if (::boost::size(current_piece) > 1)
         {
             *out++ = current_piece;
@@ -213,26 +272,14 @@ struct action_selector<overlay_intersection, RemoveSpikes>
 
     template
     <
-        typename OutputIterator,
-        typename LineStringOut,
-        typename LineString,
+        typename LineStringOrPointOut,
         typename Point,
-        typename Operation
+        typename OutputIterator
     >
-    static inline void isolated_point(LineStringOut&,
-                LineString const&,
-                segment_identifier&,
-                signed_size_type, Point const& point,
-                Operation const& , OutputIterator& out)
+    static inline void isolated_point(Point const& point,
+                                      OutputIterator& out)
     {
-        LineStringOut isolated_point_ls;
-        geometry::append(isolated_point_ls, point);
-
-#ifndef BOOST_GEOMETRY_ALLOW_ONE_POINT_LINESTRINGS
-        geometry::append(isolated_point_ls, point);
-#endif // BOOST_GEOMETRY_ALLOW_ONE_POINT_LINESTRINGS
-
-        *out++ = isolated_point_ls;
+        add_isolated_point<LineStringOrPointOut>::apply(point, out);
     }
 
     static inline bool is_entered(bool entered)
@@ -251,7 +298,7 @@ struct action_selector<overlay_intersection, RemoveSpikes>
 template <bool RemoveSpikes>
 struct action_selector<overlay_difference, RemoveSpikes>
 {
-    typedef action_selector<overlay_intersection, RemoveSpikes> normal_action;
+    using normal_action = action_selector<overlay_intersection, RemoveSpikes>;
 
     template
     <
@@ -260,20 +307,18 @@ struct action_selector<overlay_difference, RemoveSpikes>
         typename LineString,
         typename Point,
         typename Operation,
-        typename SideStrategy,
-        typename RobustPolicy
+        typename Strategy
     >
     static inline void enter(LineStringOut& current_piece,
                 LineString const& linestring,
                 segment_identifier& segment_id,
                 signed_size_type index, Point const& point,
                 Operation const& operation,
-                SideStrategy const& strategy,
-                RobustPolicy const& robust_policy,
+                Strategy const& strategy,
                 OutputIterator& out)
     {
         normal_action::leave(current_piece, linestring, segment_id, index,
-                    point, operation, strategy, robust_policy, out);
+                    point, operation, strategy, out);
     }
 
     template
@@ -283,35 +328,28 @@ struct action_selector<overlay_difference, RemoveSpikes>
         typename LineString,
         typename Point,
         typename Operation,
-        typename SideStrategy,
-        typename RobustPolicy
+        typename Strategy
     >
     static inline void leave(LineStringOut& current_piece,
                 LineString const& linestring,
                 segment_identifier& segment_id,
                 signed_size_type index, Point const& point,
                 Operation const& operation,
-                SideStrategy const& strategy,
-                RobustPolicy const& robust_policy,
+                Strategy const& strategy,
                 OutputIterator& out)
     {
         normal_action::enter(current_piece, linestring, segment_id, index,
-                    point, operation, strategy, robust_policy, out);
+                    point, operation, strategy, out);
     }
 
     template
     <
-        typename OutputIterator,
-        typename LineStringOut,
-        typename LineString,
+        typename LineStringOrPointOut,
         typename Point,
-        typename Operation
+        typename OutputIterator
     >
-    static inline void isolated_point(LineStringOut&,
-                LineString const&,
-                segment_identifier&,
-                signed_size_type, Point const&,
-                Operation const&, OutputIterator&)
+    static inline void isolated_point(Point const&,
+                                      OutputIterator const&)
     {
     }
 
@@ -327,7 +365,7 @@ struct action_selector<overlay_difference, RemoveSpikes>
 
 };
 
-}
+} // namespace following
 
 /*!
 \brief Follows a linestring from intersection point to intersection point, outputting which
@@ -336,64 +374,23 @@ struct action_selector<overlay_difference, RemoveSpikes>
  */
 template
 <
-    typename LineStringOut,
+    typename GeometryOut,
     typename LineString,
     typename Polygon,
     overlay_type OverlayType,
-    bool RemoveSpikes = true
+    bool RemoveSpikes,
+    bool FollowIsolatedPoints
 >
 class follow
 {
-
-#ifdef BOOST_GEOMETRY_SETOPS_LA_OLD_BEHAVIOR
-    template <typename Turn>
-    struct sort_on_segment
-    {
-        // In case of turn point at the same location, we want to have continue/blocked LAST
-        // because that should be followed (intersection) or skipped (difference).
-        inline int operation_order(Turn const& turn) const
-        {
-            operation_type const& operation = turn.operations[0].operation;
-            switch(operation)
-            {
-                case operation_opposite : return 0;
-                case operation_none : return 0;
-                case operation_union : return 1;
-                case operation_intersection : return 2;
-                case operation_blocked : return 3;
-                case operation_continue : return 4;
-            }
-            return -1;
-        };
-
-        inline bool use_operation(Turn const& left, Turn const& right) const
-        {
-            // If they are the same, OK.
-            return operation_order(left) < operation_order(right);
-        }
-
-        inline bool use_distance(Turn const& left, Turn const& right) const
-        {
-            return left.operations[0].fraction == right.operations[0].fraction
-                ? use_operation(left, right)
-                : left.operations[0].fraction < right.operations[0].fraction
-                ;
-        }
-
-        inline bool operator()(Turn const& left, Turn const& right) const
-        {
-            segment_identifier const& sl = left.operations[0].seg_id;
-            segment_identifier const& sr = right.operations[0].seg_id;
-
-            return sl == sr
-                ? use_distance(left, right)
-                : sl < sr
-                ;
-
-        }
-    };
-#endif // BOOST_GEOMETRY_SETOPS_LA_OLD_BEHAVIOR
-
+    using linear = geometry::detail::output_geometry_access
+        <
+            GeometryOut, linestring_tag, linestring_tag
+        >;
+    using pointlike = geometry::detail::output_geometry_access
+        <
+            GeometryOut, point_tag, linestring_tag
+        >;
 
 public :
 
@@ -409,85 +406,80 @@ public :
     <
         typename Turns,
         typename OutputIterator,
-        typename RobustPolicy,
         typename Strategy
     >
     static inline OutputIterator apply(LineString const& linestring, Polygon const& polygon,
                 detail::overlay::operation_type ,  // TODO: this parameter might be redundant
                 Turns& turns,
-                RobustPolicy const& robust_policy,
                 OutputIterator out,
                 Strategy const& strategy)
     {
-        typedef typename boost::range_iterator<Turns>::type turn_iterator;
-        typedef typename boost::range_value<Turns>::type turn_type;
-        typedef typename boost::range_iterator
-            <
-                typename turn_type::container_type
-            >::type turn_operation_iterator_type;
-
-        typedef following::action_selector<OverlayType, RemoveSpikes> action;
-
-        typename Strategy::template point_in_geometry_strategy
-            <
-                LineString, Polygon
-            >::type const pt_in_poly_strategy
-            = strategy.template get_point_in_geometry_strategy<LineString, Polygon>();
+        using action = following::action_selector<OverlayType, RemoveSpikes>;
 
         // Sort intersection points on segments-along-linestring, and distance
         // (like in enrich is done for poly/poly)
         // sort turns by Linear seg_id, then by fraction, then
         // for same ring id: x, u, i, c
         // for different ring id: c, i, u, x
-#ifdef BOOST_GEOMETRY_SETOPS_LA_OLD_BEHAVIOR
-        std::sort(boost::begin(turns), boost::end(turns), sort_on_segment<turn_type>());
-#else
-        typedef relate::turns::less<0, relate::turns::less_op_linear_areal_single<0> > turn_less;
+        using turn_less = relate::turns::less
+            <
+                0, relate::turns::less_op_linear_areal_single<0>, Strategy
+            >;
         std::sort(boost::begin(turns), boost::end(turns), turn_less());
-#endif
 
-        LineStringOut current_piece;
+        typename linear::type current_piece;
         geometry::segment_identifier current_segment_id(0, -1, -1, -1);
 
         // Iterate through all intersection points (they are ordered along the line)
         bool entered = false;
         bool first = true;
-        for (turn_iterator it = boost::begin(turns); it != boost::end(turns); ++it)
+        for (auto const& turn : turns)
         {
-            turn_operation_iterator_type iit = boost::begin(it->operations);
+            auto const& op = turn.operations[0];
 
-            if (following::was_entered(*it, *iit, first, linestring, polygon, pt_in_poly_strategy))
+            if (following::was_entered(turn, op, first, linestring, polygon, strategy))
             {
-                debug_traverse(*it, *iit, "-> Was entered");
+                debug_traverse(turn, op, "-> Was entered");
                 entered = true;
             }
 
-            if (following::is_staying_inside(*it, *iit, entered, first, linestring, polygon, pt_in_poly_strategy))
+            if (following::is_staying_inside(turn, op, entered, first, linestring, polygon, strategy))
             {
-                debug_traverse(*it, *iit, "-> Staying inside");
+                debug_traverse(turn, op, "-> Staying inside");
 
                 entered = true;
             }
-            else if (following::is_entering(*it, *iit))
+            else if (following::is_entering(turn, op))
             {
-                debug_traverse(*it, *iit, "-> Entering");
+                debug_traverse(turn, op, "-> Entering");
 
                 entered = true;
                 action::enter(current_piece, linestring, current_segment_id,
-                    iit->seg_id.segment_index, it->point, *iit,
-                    strategy, robust_policy,
-                    out);
+                    op.seg_id.segment_index, turn.point, op,
+                    strategy,
+                    linear::get(out));
             }
-            else if (following::is_leaving(*it, *iit, entered, first, linestring, polygon, pt_in_poly_strategy))
+            else if (following::is_leaving(turn, op, entered, first, linestring, polygon, strategy))
             {
-                debug_traverse(*it, *iit, "-> Leaving");
+                debug_traverse(turn, op, "-> Leaving");
 
                 entered = false;
                 action::leave(current_piece, linestring, current_segment_id,
-                    iit->seg_id.segment_index, it->point, *iit,
-                    strategy, robust_policy,
-                    out);
+                    op.seg_id.segment_index, turn.point, op,
+                    strategy,
+                    linear::get(out));
             }
+            else if (BOOST_GEOMETRY_CONDITION(FollowIsolatedPoints)
+                  && following::is_touching(turn, op, entered))
+            {
+                debug_traverse(turn, op, "-> Isolated point");
+
+                action::template isolated_point
+                    <
+                        typename pointlike::type
+                    >(turn.point, pointlike::get(out));
+            }
+
             first = false;
         }
 
@@ -499,15 +491,25 @@ public :
                 >::apply(linestring,
                          current_segment_id,
                          static_cast<signed_size_type>(boost::size(linestring) - 1),
-                         strategy, robust_policy,
+                         strategy,
                          current_piece);
         }
 
         // Output the last one, if applicable
-        if (::boost::size(current_piece) > 1)
+        std::size_t current_piece_size = ::boost::size(current_piece);
+        if (current_piece_size > 1)
         {
-            *out++ = current_piece;
+            *linear::get(out)++ = current_piece;
         }
+        else if (BOOST_GEOMETRY_CONDITION(FollowIsolatedPoints)
+              && current_piece_size == 1)
+        {
+            action::template isolated_point
+                <
+                    typename pointlike::type
+                >(range::front(current_piece), pointlike::get(out));
+        }
+
         return out;
     }
 
