@@ -10,13 +10,20 @@
 #pragma once
 #endif
 
+#include <boost/math/tools/config.hpp>
+
+// TODO(mborland): Need to remove recurrsion from these algos
+#ifndef BOOST_MATH_HAS_NVRTC
+
 #include <boost/math/special_functions/math_fwd.hpp>
 #include <boost/math/policies/error_handling.hpp>
 #include <boost/math/special_functions/fpclassify.hpp>
 #include <boost/math/special_functions/sign.hpp>
 #include <boost/math/special_functions/trunc.hpp>
+#include <boost/math/tools/traits.hpp>
+#include <type_traits>
+#include <cfloat>
 
-#include <float.h>
 
 #if !defined(_CRAYC) && !defined(__CUDACC__) && (!defined(__GNUC__) || (__GNUC__ > 3) || ((__GNUC__ == 3) && (__GNUC_MINOR__ > 3)))
 #if (defined(_M_IX86_FP) && (_M_IX86_FP >= 2)) || defined(__SSE2__)
@@ -29,8 +36,8 @@ namespace boost{ namespace math{
 
    namespace concepts {
 
-      struct real_concept;
-      struct std_real_concept;
+      class real_concept;
+      class std_real_concept;
 
    }
 
@@ -39,49 +46,49 @@ namespace detail{
 template <class T>
 struct has_hidden_guard_digits;
 template <>
-struct has_hidden_guard_digits<float> : public mpl::false_ {};
+struct has_hidden_guard_digits<float> : public std::false_type {};
 template <>
-struct has_hidden_guard_digits<double> : public mpl::false_ {};
+struct has_hidden_guard_digits<double> : public std::false_type {};
 template <>
-struct has_hidden_guard_digits<long double> : public mpl::false_ {};
+struct has_hidden_guard_digits<long double> : public std::false_type {};
 #ifdef BOOST_HAS_FLOAT128
 template <>
-struct has_hidden_guard_digits<__float128> : public mpl::false_ {};
+struct has_hidden_guard_digits<__float128> : public std::false_type {};
 #endif
 template <>
-struct has_hidden_guard_digits<boost::math::concepts::real_concept> : public mpl::false_ {};
+struct has_hidden_guard_digits<boost::math::concepts::real_concept> : public std::false_type {};
 template <>
-struct has_hidden_guard_digits<boost::math::concepts::std_real_concept> : public mpl::false_ {};
+struct has_hidden_guard_digits<boost::math::concepts::std_real_concept> : public std::false_type {};
 
 template <class T, bool b>
-struct has_hidden_guard_digits_10 : public mpl::false_ {};
+struct has_hidden_guard_digits_10 : public std::false_type {};
 template <class T>
-struct has_hidden_guard_digits_10<T, true> : public mpl::bool_<(std::numeric_limits<T>::digits10 != std::numeric_limits<T>::max_digits10)> {};
+struct has_hidden_guard_digits_10<T, true> : public std::integral_constant<bool, (std::numeric_limits<T>::digits10 != std::numeric_limits<T>::max_digits10)> {};
 
 template <class T>
-struct has_hidden_guard_digits 
-   : public has_hidden_guard_digits_10<T, 
+struct has_hidden_guard_digits
+   : public has_hidden_guard_digits_10<T,
    std::numeric_limits<T>::is_specialized
    && (std::numeric_limits<T>::radix == 10) >
 {};
 
 template <class T>
-inline const T& normalize_value(const T& val, const mpl::false_&) { return val; }
+inline const T& normalize_value(const T& val, const std::false_type&) { return val; }
 template <class T>
-inline T normalize_value(const T& val, const mpl::true_&) 
+inline T normalize_value(const T& val, const std::true_type&)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
-   boost::intmax_t shift = std::numeric_limits<T>::digits - ilogb(val) - 1;
+   std::intmax_t shift = (std::intmax_t)std::numeric_limits<T>::digits - (std::intmax_t)ilogb(val) - 1;
    T result = scalbn(val, shift);
    result = round(result);
-   return scalbn(result, -shift); 
+   return scalbn(result, -shift);
 }
 
 template <class T>
-inline T get_smallest_value(mpl::true_ const&)
-{
+inline T get_smallest_value(std::true_type const&) {
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
    //
    // numeric_limits lies about denorms being present - particularly
    // when this can be turned on or off at runtime, as is the case
@@ -89,14 +96,14 @@ inline T get_smallest_value(mpl::true_ const&)
    //
    static const T m = std::numeric_limits<T>::denorm_min();
 #ifdef BOOST_MATH_CHECK_SSE2
-   return (_mm_getcsr() & (_MM_FLUSH_ZERO_ON | 0x40)) ? tools::min_value<T>() : m;;
+   return (_mm_getcsr() & (_MM_FLUSH_ZERO_ON | 0x40)) ? tools::min_value<T>() : m;
 #else
    return ((tools::min_value<T>() / 2) == 0) ? tools::min_value<T>() : m;
 #endif
 }
 
 template <class T>
-inline T get_smallest_value(mpl::false_ const&)
+inline T get_smallest_value(std::false_type const&)
 {
    return tools::min_value<T>();
 }
@@ -104,11 +111,12 @@ inline T get_smallest_value(mpl::false_ const&)
 template <class T>
 inline T get_smallest_value()
 {
-#if defined(BOOST_MSVC) && (BOOST_MSVC <= 1310)
-   return get_smallest_value<T>(mpl::bool_<std::numeric_limits<T>::is_specialized && (std::numeric_limits<T>::has_denorm == 1)>());
-#else
-   return get_smallest_value<T>(mpl::bool_<std::numeric_limits<T>::is_specialized && (std::numeric_limits<T>::has_denorm == std::denorm_present)>());
-#endif
+   return get_smallest_value<T>(std::integral_constant<bool, std::numeric_limits<T>::is_specialized>());
+}
+
+template <class T>
+inline bool has_denorm_now() {
+   return get_smallest_value<T>() < tools::min_value<T>();
 }
 
 //
@@ -119,41 +127,16 @@ template <class T>
 T get_min_shift_value();
 
 template <class T>
-struct min_shift_initializer
-{
-   struct init
-   {
-      init()
-      {
-         do_init();
-      }
-      static void do_init()
-      {
-         get_min_shift_value<T>();
-      }
-      void force_instantiate()const{}
-   };
-   static const init initializer;
-   static void force_instantiate()
-   {
-      initializer.force_instantiate();
-   }
-};
-
-template <class T>
-const typename min_shift_initializer<T>::init min_shift_initializer<T>::initializer;
-
-template <class T>
-inline T calc_min_shifted(const mpl::true_&)
+inline T calc_min_shifted(const std::true_type&)
 {
    BOOST_MATH_STD_USING
    return ldexp(tools::min_value<T>(), tools::digits<T>() + 1);
 }
 template <class T>
-inline T calc_min_shifted(const mpl::false_&)
+inline T calc_min_shifted(const std::false_type&)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
    return scalbn(tools::min_value<T>(), std::numeric_limits<T>::digits + 1);
 }
@@ -162,32 +145,48 @@ inline T calc_min_shifted(const mpl::false_&)
 template <class T>
 inline T get_min_shift_value()
 {
-   static const T val = calc_min_shifted<T>(mpl::bool_<!std::numeric_limits<T>::is_specialized || std::numeric_limits<T>::radix == 2>());
-   min_shift_initializer<T>::force_instantiate();
-
+   static const T val = calc_min_shifted<T>(std::integral_constant<bool, !std::numeric_limits<T>::is_specialized || std::numeric_limits<T>::radix == 2>());
    return val;
 }
 
-template <class T, class Policy>
-T float_next_imp(const T& val, const mpl::true_&, const Policy& pol)
+template <class T, bool b = boost::math::tools::detail::has_backend_type<T>::value>
+struct exponent_type
 {
+   typedef int type;
+};
+
+template <class T>
+struct exponent_type<T, true>
+{
+   typedef typename T::backend_type::exponent_type type;
+};
+
+template <class T, class Policy>
+T float_next_imp(const T& val, const std::true_type&, const Policy& pol)
+{
+   typedef typename exponent_type<T>::type exponent_type;
+
    BOOST_MATH_STD_USING
-   int expon;
+   exponent_type expon;
    static const char* function = "float_next<%1%>(%1%)";
 
    int fpclass = (boost::math::fpclassify)(val);
 
-   if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
+   if (fpclass == (int)FP_INFINITE)
    {
-      if(val < 0)
+      if (val < 0)
          return -tools::max_value<T>();
+      return val;  // +INF
+   }
+   else if (fpclass == (int)FP_NAN)
+   {
       return policies::raise_domain_error<T>(
          function,
          "Argument must be finite, but got %1%", val, pol);
    }
 
    if(val >= tools::max_value<T>())
-      return policies::raise_overflow_error<T>(function, 0, pol);
+      return policies::raise_overflow_error<T>(function, nullptr, pol);
 
    if(val == 0)
       return detail::get_smallest_value<T>();
@@ -213,28 +212,34 @@ T float_next_imp(const T& val, const mpl::true_&, const Policy& pol)
 // Special version for some base other than 2:
 //
 template <class T, class Policy>
-T float_next_imp(const T& val, const mpl::false_&, const Policy& pol)
+T float_next_imp(const T& val, const std::false_type&, const Policy& pol)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   typedef typename exponent_type<T>::type exponent_type;
+
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
    BOOST_MATH_STD_USING
-   boost::intmax_t expon;
+   exponent_type expon;
    static const char* function = "float_next<%1%>(%1%)";
 
    int fpclass = (boost::math::fpclassify)(val);
 
-   if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
+   if (fpclass == (int)FP_INFINITE)
    {
-      if(val < 0)
+      if (val < 0)
          return -tools::max_value<T>();
+      return val;  // +INF
+   }
+   else if (fpclass == (int)FP_NAN)
+   {
       return policies::raise_domain_error<T>(
          function,
          "Argument must be finite, but got %1%", val, pol);
    }
 
    if(val >= tools::max_value<T>())
-      return policies::raise_overflow_error<T>(function, 0, pol);
+      return policies::raise_overflow_error<T>(function, nullptr, pol);
 
    if(val == 0)
       return detail::get_smallest_value<T>();
@@ -264,7 +269,7 @@ template <class T, class Policy>
 inline typename tools::promote_args<T>::type float_next(const T& val, const Policy& pol)
 {
    typedef typename tools::promote_args<T>::type result_type;
-   return detail::float_next_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), mpl::bool_<!std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
+   return detail::float_next_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), std::integral_constant<bool, !std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
 }
 
 #if 0 //def BOOST_MSVC
@@ -284,7 +289,7 @@ inline double float_next(const double& val, const Policy& pol)
          "Argument must be finite, but got %1%", val, pol);
 
    if(val >= tools::max_value<double>())
-      return policies::raise_overflow_error<double>(function, 0, pol);
+      return policies::raise_overflow_error<double>(function, nullptr, pol);
 
    return ::_nextafter(val, tools::max_value<double>());
 }
@@ -299,25 +304,31 @@ inline typename tools::promote_args<T>::type float_next(const T& val)
 namespace detail{
 
 template <class T, class Policy>
-T float_prior_imp(const T& val, const mpl::true_&, const Policy& pol)
+T float_prior_imp(const T& val, const std::true_type&, const Policy& pol)
 {
+   typedef typename exponent_type<T>::type exponent_type;
+
    BOOST_MATH_STD_USING
-   int expon;
+   exponent_type expon;
    static const char* function = "float_prior<%1%>(%1%)";
 
    int fpclass = (boost::math::fpclassify)(val);
 
-   if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
+   if (fpclass == (int)FP_INFINITE)
    {
-      if(val > 0)
+      if (val > 0)
          return tools::max_value<T>();
+      return val; // -INF
+   }
+   else if (fpclass == (int)FP_NAN)
+   {
       return policies::raise_domain_error<T>(
          function,
          "Argument must be finite, but got %1%", val, pol);
    }
 
    if(val <= -tools::max_value<T>())
-      return -policies::raise_overflow_error<T>(function, 0, pol);
+      return -policies::raise_overflow_error<T>(function, nullptr, pol);
 
    if(val == 0)
       return -detail::get_smallest_value<T>();
@@ -344,28 +355,34 @@ T float_prior_imp(const T& val, const mpl::true_&, const Policy& pol)
 // Special version for bases other than 2:
 //
 template <class T, class Policy>
-T float_prior_imp(const T& val, const mpl::false_&, const Policy& pol)
+T float_prior_imp(const T& val, const std::false_type&, const Policy& pol)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   typedef typename exponent_type<T>::type exponent_type;
+
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
    BOOST_MATH_STD_USING
-   boost::intmax_t expon;
+   exponent_type expon;
    static const char* function = "float_prior<%1%>(%1%)";
 
    int fpclass = (boost::math::fpclassify)(val);
 
-   if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
+   if (fpclass == (int)FP_INFINITE)
    {
-      if(val > 0)
+      if (val > 0)
          return tools::max_value<T>();
+      return val; // -INF
+   }
+   else if (fpclass == (int)FP_NAN)
+   {
       return policies::raise_domain_error<T>(
          function,
          "Argument must be finite, but got %1%", val, pol);
    }
 
    if(val <= -tools::max_value<T>())
-      return -policies::raise_overflow_error<T>(function, 0, pol);
+      return -policies::raise_overflow_error<T>(function, nullptr, pol);
 
    if(val == 0)
       return -detail::get_smallest_value<T>();
@@ -396,7 +413,7 @@ template <class T, class Policy>
 inline typename tools::promote_args<T>::type float_prior(const T& val, const Policy& pol)
 {
    typedef typename tools::promote_args<T>::type result_type;
-   return detail::float_prior_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), mpl::bool_<!std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
+   return detail::float_prior_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), std::integral_constant<bool, !std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
 }
 
 #if 0 //def BOOST_MSVC
@@ -416,7 +433,7 @@ inline double float_prior(const double& val, const Policy& pol)
          "Argument must be finite, but got %1%", val, pol);
 
    if(val <= -tools::max_value<double>())
-      return -policies::raise_overflow_error<double>(function, 0, pol);
+      return -policies::raise_overflow_error<double>(function, nullptr, pol);
 
    return ::_nextafter(val, -tools::max_value<double>());
 }
@@ -444,7 +461,7 @@ inline typename tools::promote_args<T, U>::type nextafter(const T& val, const U&
 namespace detail{
 
 template <class T, class Policy>
-T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& pol)
+T float_distance_imp(const T& a, const T& b, const std::true_type&, const Policy& pol)
 {
    BOOST_MATH_STD_USING
    //
@@ -452,13 +469,9 @@ T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& po
    //
    static const char* function = "float_distance<%1%>(%1%, %1%)";
    if(!(boost::math::isfinite)(a))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument a must be finite, but got %1%", a, pol);
+      return policies::raise_domain_error<T>(function, "Argument a must be finite, but got %1%", a, pol);
    if(!(boost::math::isfinite)(b))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument b must be finite, but got %1%", b, pol);
+      return policies::raise_domain_error<T>(function, "Argument b must be finite, but got %1%", b, pol);
    //
    // Special cases:
    //
@@ -475,13 +488,13 @@ T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& po
          + fabs(float_distance(static_cast<T>((a < 0) ? T(-detail::get_smallest_value<T>()) : detail::get_smallest_value<T>()), a, pol));
    //
    // By the time we get here, both a and b must have the same sign, we want
-   // b > a and both postive for the following logic:
+   // b > a and both positive for the following logic:
    //
    if(a < 0)
       return float_distance(static_cast<T>(-b), static_cast<T>(-a), pol);
 
-   BOOST_ASSERT(a >= 0);
-   BOOST_ASSERT(b >= a);
+   BOOST_MATH_ASSERT(a >= 0);
+   BOOST_MATH_ASSERT(b >= a);
 
    int expon;
    //
@@ -489,7 +502,7 @@ T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& po
    // because we actually have fewer than tools::digits<T>()
    // significant bits in the representation:
    //
-   frexp(((boost::math::fpclassify)(a) == (int)FP_SUBNORMAL) ? tools::min_value<T>() : a, &expon);
+   (void)frexp(((boost::math::fpclassify)(a) == (int)FP_SUBNORMAL) ? tools::min_value<T>() : a, &expon);
    T upper = ldexp(T(1), expon);
    T result = T(0);
    //
@@ -499,7 +512,7 @@ T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& po
    if(b > upper)
    {
       int expon2;
-      frexp(b, &expon2);
+      (void)frexp(b, &expon2);
       T upper2 = ldexp(T(0.5), expon2);
       result = float_distance(upper2, b);
       result += (expon2 - expon - 1) * ldexp(T(1), tools::digits<T>() - 1);
@@ -542,17 +555,17 @@ T float_distance_imp(const T& a, const T& b, const mpl::true_&, const Policy& po
    //
    // Result must be an integer:
    //
-   BOOST_ASSERT(result == floor(result));
+   BOOST_MATH_ASSERT(result == floor(result));
    return result;
 } // float_distance_imp
 //
 // Special versions for bases other than 2:
 //
 template <class T, class Policy>
-T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& pol)
+T float_distance_imp(const T& a, const T& b, const std::false_type&, const Policy& pol)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
    BOOST_MATH_STD_USING
    //
@@ -560,13 +573,9 @@ T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& p
    //
    static const char* function = "float_distance<%1%>(%1%, %1%)";
    if(!(boost::math::isfinite)(a))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument a must be finite, but got %1%", a, pol);
+      return policies::raise_domain_error<T>(function, "Argument a must be finite, but got %1%", a, pol);
    if(!(boost::math::isfinite)(b))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument b must be finite, but got %1%", b, pol);
+      return policies::raise_domain_error<T>(function, "Argument b must be finite, but got %1%", b, pol);
    //
    // Special cases:
    //
@@ -583,15 +592,15 @@ T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& p
          + fabs(float_distance(static_cast<T>((a < 0) ? T(-detail::get_smallest_value<T>()) : detail::get_smallest_value<T>()), a, pol));
    //
    // By the time we get here, both a and b must have the same sign, we want
-   // b > a and both postive for the following logic:
+   // b > a and both positive for the following logic:
    //
    if(a < 0)
       return float_distance(static_cast<T>(-b), static_cast<T>(-a), pol);
 
-   BOOST_ASSERT(a >= 0);
-   BOOST_ASSERT(b >= a);
+   BOOST_MATH_ASSERT(a >= 0);
+   BOOST_MATH_ASSERT(b >= a);
 
-   boost::intmax_t expon;
+   std::intmax_t expon;
    //
    // Note that if a is a denorm then the usual formula fails
    // because we actually have fewer than tools::digits<T>()
@@ -606,7 +615,7 @@ T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& p
    //
    if(b > upper)
    {
-      boost::intmax_t expon2 = 1 + ilogb(b);
+      std::intmax_t expon2 = 1 + ilogb(b);
       T upper2 = scalbn(T(1), expon2 - 1);
       result = float_distance(upper2, b);
       result += (expon2 - expon - 1) * scalbn(T(1), std::numeric_limits<T>::digits - 1);
@@ -649,7 +658,7 @@ T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& p
    //
    // Result must be an integer:
    //
-   BOOST_ASSERT(result == floor(result));
+   BOOST_MATH_ASSERT(result == floor(result));
    return result;
 } // float_distance_imp
 
@@ -658,8 +667,35 @@ T float_distance_imp(const T& a, const T& b, const mpl::false_&, const Policy& p
 template <class T, class U, class Policy>
 inline typename tools::promote_args<T, U>::type float_distance(const T& a, const U& b, const Policy& pol)
 {
-   typedef typename tools::promote_args<T, U>::type result_type;
-   return detail::float_distance_imp(detail::normalize_value(static_cast<result_type>(a), typename detail::has_hidden_guard_digits<result_type>::type()), detail::normalize_value(static_cast<result_type>(b), typename detail::has_hidden_guard_digits<result_type>::type()), mpl::bool_<!std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
+   //
+   // We allow ONE of a and b to be an integer type, otherwise both must be the SAME type.
+   //
+   static_assert(
+      (std::is_same<T, U>::value
+      || (std::is_integral<T>::value && !std::is_integral<U>::value)
+      || (!std::is_integral<T>::value && std::is_integral<U>::value)
+      || (std::numeric_limits<T>::is_specialized && std::numeric_limits<U>::is_specialized
+         && (std::numeric_limits<T>::digits == std::numeric_limits<U>::digits)
+         && (std::numeric_limits<T>::radix == std::numeric_limits<U>::radix)
+         && !std::numeric_limits<T>::is_integer && !std::numeric_limits<U>::is_integer)),
+      "Float distance between two different floating point types is undefined.");
+
+   BOOST_MATH_IF_CONSTEXPR (!std::is_same<T, U>::value)
+   {
+      BOOST_MATH_IF_CONSTEXPR(std::is_integral<T>::value)
+      {
+         return float_distance(static_cast<U>(a), b, pol);
+      }
+      else
+      {
+         return float_distance(a, static_cast<T>(b), pol);
+      }
+   }
+   else
+   {
+      typedef typename tools::promote_args<T, U>::type result_type;
+      return detail::float_distance_imp(detail::normalize_value(static_cast<result_type>(a), typename detail::has_hidden_guard_digits<result_type>::type()), detail::normalize_value(static_cast<result_type>(b), typename detail::has_hidden_guard_digits<result_type>::type()), std::integral_constant<bool, !std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
+   }
 }
 
 template <class T, class U>
@@ -671,7 +707,7 @@ typename tools::promote_args<T, U>::type float_distance(const T& a, const U& b)
 namespace detail{
 
 template <class T, class Policy>
-T float_advance_imp(T val, int distance, const mpl::true_&, const Policy& pol)
+T float_advance_imp(T val, int distance, const std::true_type&, const Policy& pol)
 {
    BOOST_MATH_STD_USING
    //
@@ -682,9 +718,7 @@ T float_advance_imp(T val, int distance, const mpl::true_&, const Policy& pol)
    int fpclass = (boost::math::fpclassify)(val);
 
    if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument val must be finite, but got %1%", val, pol);
+      return policies::raise_domain_error<T>(function, "Argument val must be finite, but got %1%", val, pol);
 
    if(val < 0)
       return -float_advance(-val, -distance, pol);
@@ -714,12 +748,10 @@ T float_advance_imp(T val, int distance, const mpl::true_&, const Policy& pol)
    }
 
    int expon;
-   frexp(val, &expon);
+   (void)frexp(val, &expon);
    T limit = ldexp((distance < 0 ? T(0.5f) : T(1)), expon);
-   if(val <= tools::min_value<T>())
-   {
-      limit = sign(T(distance)) * tools::min_value<T>();
-   }
+   // We can not have denorms here, since we have taken care of them above:
+   BOOST_MATH_ASSERT(val > tools::min_value<T>());
    T limit_distance = float_distance(val, limit);
    while(fabs(limit_distance) < abs(distance))
    {
@@ -738,7 +770,7 @@ T float_advance_imp(T val, int distance, const mpl::true_&, const Policy& pol)
       limit_distance = float_distance(val, limit);
       if(distance && (limit_distance == 0))
       {
-         return policies::raise_evaluation_error<T>(function, "Internal logic failed while trying to increment floating point value %1%: most likely your FPU is in non-IEEE conforming mode.", val, pol);
+         return policies::raise_evaluation_error<T>(function, "Internal logic failed while trying to increment floating point value %1%: most likely your FPU is in non-IEEE conforming mode.", val, pol);  // LCOV_EXCL_LINE This *should* be unreachable.
       }
    }
    if((0.5f == frexp(val, &expon)) && (distance < 0))
@@ -747,17 +779,17 @@ T float_advance_imp(T val, int distance, const mpl::true_&, const Policy& pol)
    if(val != 0)
       diff = distance * ldexp(T(1), expon - tools::digits<T>());
    if(diff == 0)
-      diff = distance * detail::get_smallest_value<T>();
+      diff = distance * detail::get_smallest_value<T>(); // LCOV_EXCL_LINE This *should* be unreachable given that denorms are handled above already.
    return val += diff;
 } // float_advance_imp
 //
 // Special version for bases other than 2:
 //
 template <class T, class Policy>
-T float_advance_imp(T val, int distance, const mpl::false_&, const Policy& pol)
+T float_advance_imp(T val, int distance, const std::false_type&, const Policy& pol)
 {
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::is_specialized);
-   BOOST_STATIC_ASSERT(std::numeric_limits<T>::radix != 2);
+   static_assert(std::numeric_limits<T>::is_specialized, "Type T must be specialized.");
+   static_assert(std::numeric_limits<T>::radix != 2, "Type T must be specialized.");
 
    BOOST_MATH_STD_USING
    //
@@ -768,9 +800,7 @@ T float_advance_imp(T val, int distance, const mpl::false_&, const Policy& pol)
    int fpclass = (boost::math::fpclassify)(val);
 
    if((fpclass == (int)FP_NAN) || (fpclass == (int)FP_INFINITE))
-      return policies::raise_domain_error<T>(
-         function,
-         "Argument val must be finite, but got %1%", val, pol);
+      return policies::raise_domain_error<T>(function, "Argument val must be finite, but got %1%", val, pol);
 
    if(val < 0)
       return -float_advance(-val, -distance, pol);
@@ -799,12 +829,9 @@ T float_advance_imp(T val, int distance, const mpl::false_&, const Policy& pol)
       return val;
    }
 
-   boost::intmax_t expon = 1 + ilogb(val);
+   std::intmax_t expon = 1 + ilogb(val);
    T limit = scalbn(T(1), distance < 0 ? expon - 1 : expon);
-   if(val <= tools::min_value<T>())
-   {
-      limit = sign(T(distance)) * tools::min_value<T>();
-   }
+   BOOST_MATH_ASSERT(val > tools::min_value<T>()); // denorms already handled.
    T limit_distance = float_distance(val, limit);
    while(fabs(limit_distance) < abs(distance))
    {
@@ -817,13 +844,13 @@ T float_advance_imp(T val, int distance, const mpl::false_&, const Policy& pol)
       }
       else
       {
-         limit *= std::numeric_limits<T>::radix;
-         expon++;
+         limit *= std::numeric_limits<T>::radix; // LCOV_EXCL_LINE Probably unreachable for the decimal types we have?
+         expon++;                                // LCOV_EXCL_LINE
       }
       limit_distance = float_distance(val, limit);
       if(distance && (limit_distance == 0))
       {
-         return policies::raise_evaluation_error<T>(function, "Internal logic failed while trying to increment floating point value %1%: most likely your FPU is in non-IEEE conforming mode.", val, pol);
+         return policies::raise_evaluation_error<T>(function, "Internal logic failed while trying to increment floating point value %1%: most likely your FPU is in non-IEEE conforming mode.", val, pol);  // LCOV_EXCL_LINE should never get here!
       }
    }
    /*expon = 1 + ilogb(val);
@@ -833,7 +860,7 @@ T float_advance_imp(T val, int distance, const mpl::false_&, const Policy& pol)
    if(val != 0)
       diff = distance * scalbn(T(1), expon - std::numeric_limits<T>::digits);
    if(diff == 0)
-      diff = distance * detail::get_smallest_value<T>();
+      diff = distance * detail::get_smallest_value<T>(); // LCOV_EXCL_LINE This *should* be unreachable given that denorms are handled above.
    return val += diff;
 } // float_advance_imp
 
@@ -843,7 +870,7 @@ template <class T, class Policy>
 inline typename tools::promote_args<T>::type float_advance(T val, int distance, const Policy& pol)
 {
    typedef typename tools::promote_args<T>::type result_type;
-   return detail::float_advance_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), distance, mpl::bool_<!std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
+   return detail::float_advance_imp(detail::normalize_value(static_cast<result_type>(val), typename detail::has_hidden_guard_digits<result_type>::type()), distance, std::integral_constant<bool, !std::numeric_limits<result_type>::is_specialized || (std::numeric_limits<result_type>::radix == 2)>(), pol);
 }
 
 template <class T>
@@ -854,5 +881,6 @@ inline typename tools::promote_args<T>::type float_advance(const T& val, int dis
 
 }} // boost math namespaces
 
-#endif // BOOST_MATH_SPECIAL_NEXT_HPP
+#endif
 
+#endif // BOOST_MATH_SPECIAL_NEXT_HPP
