@@ -7,7 +7,7 @@
  *
  * See http://www.boost.org for most recent version including documentation.
  *
- * $Id: piecewise_constant_distribution.hpp 78340 2012-05-05 21:14:37Z steven_watanabe $
+ * $Id$
  */
 
 #ifndef BOOST_RANDOM_PIECEWISE_CONSTANT_DISTRIBUTION_HPP_INCLUDED
@@ -22,13 +22,10 @@
 #include <boost/random/detail/operators.hpp>
 #include <boost/random/detail/vector_io.hpp>
 
-#ifndef BOOST_NO_0X_HDR_INITIALIZER_LIST
+#ifndef BOOST_NO_CXX11_HDR_INITIALIZER_LIST
 #include <initializer_list>
 #endif
 
-#include <boost/range/begin.hpp>
-#include <boost/range/end.hpp>
-
 namespace boost {
 namespace random {
 
@@ -84,7 +81,7 @@ public:
                 }
             }
         }
-#ifndef BOOST_NO_0X_HDR_INITIALIZER_LIST
+#ifndef BOOST_NO_CXX11_HDR_INITIALIZER_LIST
         /**
          * Constructs a @c param_type object from an
          * initializer_list containing the interval boundaries
@@ -127,8 +124,8 @@ public:
         template<class IntervalRange, class WeightRange>
         param_type(const IntervalRange& intervals_arg,
                    const WeightRange& weights_arg)
-          : _intervals(boost::begin(intervals_arg), boost::end(intervals_arg)),
-            _weights(boost::begin(weights_arg), boost::end(weights_arg))
+          : _intervals(std::begin(intervals_arg), std::end(intervals_arg)),
+            _weights(std::begin(weights_arg), std::end(weights_arg))
         {
             if(_intervals.size() < 2) {
                 _intervals.clear();
@@ -185,7 +182,7 @@ public:
             detail::print_vector(os, parm._weights);
             return os;
         }
-        
+
         /** Reads the parameters from a @c std::istream. */
         BOOST_RANDOM_DETAIL_ISTREAM_OPERATOR(is, param_type, parm)
         {
@@ -271,7 +268,7 @@ public:
             _bins.param(bins_param);
         }
     }
-#ifndef BOOST_NO_0X_HDR_INITIALIZER_LIST
+#ifndef BOOST_NO_CXX11_HDR_INITIALIZER_LIST
     /**
      * Constructs a piecewise_constant_distribution from an
      * initializer_list containing the interval boundaries
@@ -318,7 +315,7 @@ public:
     piecewise_constant_distribution(const IntervalsRange& intervals_arg,
                                     const WeightsRange& weights_arg)
       : _bins(weights_arg),
-        _intervals(boost::begin(intervals_arg), boost::end(intervals_arg))
+        _intervals(std::begin(intervals_arg), std::end(intervals_arg))
     {
         if(_intervals.size() < 2) {
             _intervals.clear();
@@ -367,7 +364,7 @@ public:
         std::size_t i = _bins(urng);
         return uniform_real<RealType>(_intervals[i], _intervals[i+1])(urng);
     }
-    
+
     /**
      * Returns a value distributed according to the parameters
      * specified by param.
@@ -377,7 +374,7 @@ public:
     {
         return piecewise_constant_distribution(parm)(urng);
     }
-    
+
     /** Returns the smallest value that the distribution can produce. */
     result_type min BOOST_PREVENT_MACRO_SUBSTITUTION () const
     { return _intervals.front(); }
@@ -409,12 +406,12 @@ public:
     void param(const param_type& parm)
     {
         std::vector<RealType> new_intervals(parm._intervals);
-        typedef discrete_distribution<std::size_t, RealType> bins_type;
+        typedef discrete_distribution<std::size_t, WeightType> bins_type;
         typename bins_type::param_type bins_param(parm._weights);
         _bins.param(bins_param);
         _intervals.swap(new_intervals);
     }
-    
+
     /**
      * Effects: Subsequent uses of the distribution do not depend
      * on values produced by any engine prior to invoking reset.
