@@ -2,13 +2,14 @@
 // composed_7.cpp
 // ~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2021 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 
 #include <boost/asio/compose.hpp>
+#include <boost/asio/deferred.hpp>
 #include <boost/asio/io_context.hpp>
 #include <boost/asio/ip/tcp.hpp>
 #include <boost/asio/steady_timer.hpp>
@@ -56,7 +57,7 @@ struct async_write_messages_implementation
   // A steady timer used for introducing a delay.
   std::unique_ptr<boost::asio::steady_timer> delay_timer_;
 
-  // To manage the cycle between the multiple underlying asychronous
+  // To manage the cycle between the multiple underlying asynchronous
   // operations, our implementation is a state machine.
   enum { starting, waiting, writing } state_;
 
@@ -118,15 +119,26 @@ auto async_write_messages(tcp::socket& socket,
     const T& message, std::size_t repeat_count,
     CompletionToken&& token)
   // The return type of the initiating function is deduced from the combination
-  // of CompletionToken type and the completion handler's signature. When the
-  // completion token is a simple callback, the return type is always void.
-  // In this example, when the completion token is boost::asio::yield_context
-  // (used for stackful coroutines) the return type would be also be void, as
+  // of:
+  //
+  // - the CompletionToken type,
+  // - the completion handler signature, and
+  // - the asynchronous operation's initiation function object.
+  //
+  // When the completion token is a simple callback, the return type is always
+  // void. In this example, when the completion token is boost::asio::yield_context
+  // (used for stackful coroutines) the return type would also be void, as
   // there is no non-error argument to the completion handler. When the
-  // completion token is boost::asio::use_future it would be std::future<void>.
-  -> typename boost::asio::async_result<
-    typename std::decay<CompletionToken>::type,
-    void(boost::system::error_code)>::return_type
+  // completion token is boost::asio::use_future it would be std::future<void>. When
+  // the completion token is boost::asio::deferred, the return type differs for each
+  // asynchronous operation.
+  //
+  // In C++11 we deduce the type from the call to boost::asio::async_compose.
+  -> decltype(
+      boost::asio::async_compose<
+        CompletionToken, void(boost::system::error_code)>(
+          std::declval<async_write_messages_implementation>(),
+          token, socket))
 {
   // Encode the message and copy it into an allocated buffer. The buffer will
   // be maintained for the lifetime of the composed asynchronous operation.
@@ -186,6 +198,39 @@ void test_callback()
 
 //------------------------------------------------------------------------------
 
+void test_deferred()
+{
+  boost::asio::io_context io_context;
+
+  tcp::acceptor acceptor(io_context, {tcp::v4(), 55555});
+  tcp::socket socket = acceptor.accept();
+
+  // Test our asynchronous operation using the deferred completion token. This
+  // token causes the operation's initiating function to package up the
+  // operation and its arguments to return a function object, which may then be
+  // used to launch the asynchronous operation.
+  auto op = async_write_messages(socket,
+      "Testing deferred\r\n", 5, boost::asio::deferred);
+
+  // Launch the operation using a lambda as a callback.
+  std::move(op)(
+      [](const boost::system::error_code& error)
+      {
+        if (!error)
+        {
+          std::cout << "Messages sent\n";
+        }
+        else
+        {
+          std::cout << "Error: " << error.message() << "\n";
+        }
+      });
+
+  io_context.run();
+}
+
+//------------------------------------------------------------------------------
+
 void test_future()
 {
   boost::asio::io_context io_context;
@@ -218,5 +263,6 @@ void test_future()
 int main()
 {
   test_callback();
+  test_deferred();
   test_future();
 }
