@@ -2,7 +2,7 @@
 // thread_pool.cpp
 // ~~~~~~~~~~~~~~~
 //
-// Copyright (c) 2003-2024 Christopher M. Kohlhoff (chris at kohlhoff dot com)
+// Copyright (c) 2003-2025 Christopher M. Kohlhoff (chris at kohlhoff dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -99,6 +99,40 @@ private:
 boost::asio::execution_context::id test_service::id;
 #endif // defined(BOOST_ASIO_NO_TYPEID)
 
+class test_context_service : public boost::asio::execution_context::service
+{
+public:
+  static boost::asio::execution_context::id id;
+
+  test_context_service(boost::asio::execution_context& c, int value = 0)
+    : boost::asio::execution_context::service(c),
+      value_(value)
+  {
+  }
+
+  int get_value() const
+  {
+    return value_;
+  }
+
+private:
+  virtual void shutdown() {}
+
+  int value_;
+};
+
+boost::asio::execution_context::id test_context_service::id;
+
+class test_context_service_maker :
+  public boost::asio::execution_context::service_maker
+{
+public:
+  void make(boost::asio::execution_context& ctx) const override
+  {
+    (void)boost::asio::make_service<test_context_service>(ctx, 42);
+  }
+};
+
 void thread_pool_service_test()
 {
   boost::asio::thread_pool pool1(1);
@@ -154,6 +188,14 @@ void thread_pool_service_test()
   delete svc4;
 
   BOOST_ASIO_CHECK(!boost::asio::has_service<test_service>(pool3));
+
+  // Initial service registration.
+
+  boost::asio::thread_pool pool4{1, test_context_service_maker{}};
+
+  BOOST_ASIO_CHECK(boost::asio::has_service<test_context_service>(pool4));
+  BOOST_ASIO_CHECK(boost::asio::use_service<test_context_service>(pool4).get_value()
+      == 42);
 }
 
 void thread_pool_executor_query_test()
@@ -273,6 +315,108 @@ void thread_pool_executor_execute_test()
   BOOST_ASIO_CHECK(count == 10);
 }
 
+template <typename T>
+class custom_allocator
+{
+public:
+  using value_type = T;
+
+  custom_allocator(int* live_count, int* total_count)
+    : live_count_(live_count),
+      total_count_(total_count)
+  {
+  }
+
+  template <typename U>
+  custom_allocator(const custom_allocator<U>& other) noexcept
+    : live_count_(other.live_count_),
+      total_count_(other.total_count_)
+  {
+  }
+
+  bool operator==(const custom_allocator& other) const noexcept
+  {
+    return &live_count_ == &other.live_count_ &&
+      &total_count_ == &other.total_count_;;
+  }
+
+  bool operator!=(const custom_allocator& other) const noexcept
+  {
+    return &live_count_ != &other.live_count_ ||
+      &total_count_ != &other.total_count_;
+  }
+
+  T* allocate(std::size_t n) const
+  {
+    ++(*live_count_);
+    ++(*total_count_);
+    return static_cast<T*>(::operator new(sizeof(T) * n));
+  }
+
+  void deallocate(T* p, std::size_t /*n*/) const
+  {
+    --(*live_count_);
+    ::operator delete(p);
+  }
+
+private:
+  template <typename> friend class custom_allocator;
+
+  int* live_count_;
+  int* total_count_;
+};
+
+void thread_pool_allocator_test()
+{
+  int live_count;
+  int total_count;
+
+#if !defined(BOOST_ASIO_NO_TS_EXECUTORS)
+  {
+    live_count = 0;
+    total_count = 0;
+    thread_pool pool1(std::allocator_arg,
+        custom_allocator<int>(&live_count, &total_count));
+    (void)pool1;
+
+    BOOST_ASIO_CHECK(live_count > 0);
+    BOOST_ASIO_CHECK(total_count > 0);
+  }
+
+  BOOST_ASIO_CHECK(live_count == 0);
+  BOOST_ASIO_CHECK(total_count > 0);
+#endif // !defined(BOOST_ASIO_NO_TS_EXECUTORS)
+
+  {
+    live_count = 0;
+    total_count = 0;
+    thread_pool pool2(std::allocator_arg,
+        custom_allocator<int>(&live_count, &total_count), 1);
+    (void)pool2;
+
+    BOOST_ASIO_CHECK(live_count > 0);
+    BOOST_ASIO_CHECK(total_count > 0);
+  }
+
+  BOOST_ASIO_CHECK(live_count == 0);
+  BOOST_ASIO_CHECK(total_count > 0);
+
+  {
+    live_count = 0;
+    total_count = 0;
+    thread_pool pool3(std::allocator_arg,
+        custom_allocator<int>(&live_count, &total_count), 1,
+        boost::asio::config_from_string(""));
+    (void)pool3;
+
+    BOOST_ASIO_CHECK(live_count > 0);
+    BOOST_ASIO_CHECK(total_count > 0);
+  }
+
+  BOOST_ASIO_CHECK(live_count == 0);
+  BOOST_ASIO_CHECK(total_count > 0);
+}
+
 BOOST_ASIO_TEST_SUITE
 (
   "thread_pool",
@@ -280,4 +424,5 @@ BOOST_ASIO_TEST_SUITE
   BOOST_ASIO_TEST_CASE(thread_pool_service_test)
   BOOST_ASIO_TEST_CASE(thread_pool_executor_query_test)
   BOOST_ASIO_TEST_CASE(thread_pool_executor_execute_test)
+  BOOST_ASIO_TEST_CASE(thread_pool_allocator_test)
 )
