@@ -1,4 +1,5 @@
 //  Copyright (c) 2011 Helge Bahmann
+//  Copyright (c) 2020 Andrey Semashev
 //
 //  Distributed under the Boost Software License, Version 1.0.
 //  See accompanying file LICENSE_1_0.txt or copy at
@@ -14,19 +15,25 @@
 #include <iostream>
 #include <boost/config.hpp>
 #include <boost/core/lightweight_test.hpp>
+#include "aligned_object.hpp"
 
-static const char * lock_free_level[] = {
+#if defined(BOOST_MSVC)
+// conditional expression is constant
+#pragma warning(disable: 4127)
+#endif
+
+static const char* const lock_free_level[] =
+{
     "never",
     "sometimes",
     "always"
 };
 
-template<typename T>
-void
-verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_expect)
+template< typename T >
+void verify_lock_free(const char* type_name, int lock_free_macro_val, int lock_free_expect)
 {
     BOOST_TEST(lock_free_macro_val >= 0 && lock_free_macro_val <= 2);
-    BOOST_TEST(lock_free_macro_val == lock_free_expect);
+    BOOST_TEST_EQ(lock_free_macro_val, lock_free_expect);
 
     boost::atomic<T> value;
 
@@ -35,9 +42,19 @@ verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_
     if (lock_free_macro_val == 2)
         BOOST_TEST(value.is_lock_free());
 
-    BOOST_TEST(boost::atomic<T>::is_always_lock_free == (lock_free_expect == 2));
+    BOOST_TEST_EQ(boost::atomic<T>::is_always_lock_free, (lock_free_expect == 2));
 
     std::cout << "atomic<" << type_name << "> is " << lock_free_level[lock_free_macro_val] << " lock free\n";
+
+    // atomic<T> may use larger storage than sizeof(T) to achieve lock-free property. In this case atomic_ref<T> may not be lock-free.
+    if (sizeof(boost::atomic<T>) == sizeof(T))
+    {
+        aligned_object<T, boost::atomic_ref<T>::required_alignment> object;
+        boost::atomic_ref<T> ref(object.get());
+
+        BOOST_TEST_EQ(ref.is_lock_free(), value.is_lock_free());
+        BOOST_TEST_EQ(boost::atomic_ref<T>::is_always_lock_free, boost::atomic<T>::is_always_lock_free);
+    }
 }
 
 #if (defined(__GNUC__) || defined(__SUNPRO_CC)) && defined(__i386__)
@@ -46,7 +63,8 @@ verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_
 #define EXPECT_SHORT_LOCK_FREE 2
 #define EXPECT_INT_LOCK_FREE 2
 #define EXPECT_LONG_LOCK_FREE 2
-#if defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG8B)
+#if defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG8B) || defined(__GCC_HAVE_SYNC_COMPARE_AND_SWAP_8) ||\
+    defined(__i586__) || defined(__i686__) || defined(__SSE__)
 #define EXPECT_LLONG_LOCK_FREE 2
 #else
 #define EXPECT_LLONG_LOCK_FREE 0
@@ -62,7 +80,7 @@ verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_
 #define EXPECT_INT_LOCK_FREE 2
 #define EXPECT_LONG_LOCK_FREE 2
 #define EXPECT_LLONG_LOCK_FREE 2
-#if defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG16B) && (defined(BOOST_HAS_INT128) || !defined(BOOST_NO_ALIGNMENT))
+#if defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG16B) || defined(__GCC_HAVE_SYNC_COMPARE_AND_SWAP_16)
 #define EXPECT_INT128_LOCK_FREE 2
 #else
 #define EXPECT_INT128_LOCK_FREE 0
@@ -102,14 +120,27 @@ verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_
 #define EXPECT_POINTER_LOCK_FREE 2
 #define EXPECT_BOOL_LOCK_FREE 2
 
-#elif defined(__GNUC__) &&\
+#elif defined(__GNUC__) && defined(__aarch64__)
+
+#define EXPECT_CHAR_LOCK_FREE 2
+#define EXPECT_SHORT_LOCK_FREE 2
+#define EXPECT_INT_LOCK_FREE 2
+#define EXPECT_LONG_LOCK_FREE 2
+#define EXPECT_LLONG_LOCK_FREE 2
+#define EXPECT_INT128_LOCK_FREE 2
+#define EXPECT_POINTER_LOCK_FREE 2
+#define EXPECT_BOOL_LOCK_FREE 2
+
+#elif defined(__GNUC__) && defined(__arm__) &&\
     (\
+        (defined(__ARM_ARCH) && __ARM_ARCH >= 6) ||\
         defined(__ARM_ARCH_6__)  || defined(__ARM_ARCH_6J__) ||\
         defined(__ARM_ARCH_6K__) || defined(__ARM_ARCH_6Z__) ||\
         defined(__ARM_ARCH_6ZK__) ||\
         defined(__ARM_ARCH_7__) || defined(__ARM_ARCH_7A__) ||\
         defined(__ARM_ARCH_7R__) || defined(__ARM_ARCH_7M__) ||\
-        defined(__ARM_ARCH_7EM__) || defined(__ARM_ARCH_7S__)\
+        defined(__ARM_ARCH_7EM__) || defined(__ARM_ARCH_7S__) ||\
+        defined(__ARM_ARCH_8A__)\
     )
 
 #define EXPECT_CHAR_LOCK_FREE 2
@@ -154,7 +185,7 @@ verify_lock_free(const char * type_name, int lock_free_macro_val, int lock_free_
 #define EXPECT_SHORT_LOCK_FREE 2
 #define EXPECT_INT_LOCK_FREE 2
 #define EXPECT_LONG_LOCK_FREE 2
-#if defined(_WIN64) || defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG8B) || defined(_M_AMD64) || defined(_M_IA64) || (_MSC_VER >= 1700 && defined(_M_ARM))
+#if defined(_WIN64) || defined(BOOST_ATOMIC_DETAIL_X86_HAS_CMPXCHG8B) || defined(_M_AMD64) || defined(_M_IA64) || (_MSC_VER >= 1700 && (defined(_M_ARM) || defined(_M_ARM64) || defined(_M_ARM64EC)))
 #define EXPECT_LLONG_LOCK_FREE 2
 #else
 #define EXPECT_LLONG_LOCK_FREE 0
@@ -191,6 +222,26 @@ int main(int, char *[])
     verify_lock_free<void *>("void *", BOOST_ATOMIC_POINTER_LOCK_FREE, EXPECT_SHORT_LOCK_FREE);
     verify_lock_free<bool>("bool", BOOST_ATOMIC_BOOL_LOCK_FREE, EXPECT_BOOL_LOCK_FREE);
 
+#ifndef BOOST_ATOMIC_NO_FLOATING_POINT
+
+    verify_lock_free<float>("float", BOOST_ATOMIC_FLOAT_LOCK_FREE,
+        sizeof(float) == 1 ? EXPECT_CHAR_LOCK_FREE : (sizeof(float) == 2 ? EXPECT_SHORT_LOCK_FREE :
+        (sizeof(float) <= 4 ? EXPECT_INT_LOCK_FREE : (sizeof(float) <= 8 ? EXPECT_LLONG_LOCK_FREE : (sizeof(float) <= 16 ? EXPECT_INT128_LOCK_FREE : 0)))));
+
+    verify_lock_free<double>("double", BOOST_ATOMIC_DOUBLE_LOCK_FREE,
+        sizeof(double) == 1 ? EXPECT_CHAR_LOCK_FREE : (sizeof(double) == 2 ? EXPECT_SHORT_LOCK_FREE :
+        (sizeof(double) <= 4 ? EXPECT_INT_LOCK_FREE : (sizeof(double) <= 8 ? EXPECT_LLONG_LOCK_FREE : (sizeof(double) <= 16 ? EXPECT_INT128_LOCK_FREE : 0)))));
+
+    verify_lock_free<long double>("long double", BOOST_ATOMIC_LONG_DOUBLE_LOCK_FREE,
+        sizeof(long double) == 1 ? EXPECT_CHAR_LOCK_FREE : (sizeof(long double) == 2 ? EXPECT_SHORT_LOCK_FREE :
+        (sizeof(long double) <= 4 ? EXPECT_INT_LOCK_FREE : (sizeof(long double) <= 8 ? EXPECT_LLONG_LOCK_FREE : (sizeof(long double) <= 16 ? EXPECT_INT128_LOCK_FREE : 0)))));
+
+#if defined(BOOST_HAS_INT128) && defined(BOOST_HAS_FLOAT128)
+    verify_lock_free<boost::float128_type>("float128", BOOST_ATOMIC_INT128_LOCK_FREE, EXPECT_INT128_LOCK_FREE);
+#endif
+
+#endif // BOOST_ATOMIC_NO_FLOATING_POINT
+
     bool any_lock_free =
         BOOST_ATOMIC_CHAR_LOCK_FREE > 0 ||
         BOOST_ATOMIC_SHORT_LOCK_FREE > 0 ||
