@@ -1,5 +1,5 @@
 //
-// Copyright (w) 2016-2017 Vinnie Falco (vinnie dot falco at gmail dot com)
+// Copyright (c) 2016-2019 Vinnie Falco (vinnie dot falco at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -10,6 +10,9 @@
 // Test that header file is self-contained.
 #include <boost/beast/websocket/stream.hpp>
 
+#include <boost/beast/core/tcp_stream.hpp>
+#include <boost/asio/strand.hpp>
+
 #include "test.hpp"
 
 namespace boost {
@@ -19,6 +22,46 @@ namespace websocket {
 class stream_test : public websocket_test_suite
 {
 public:
+    void
+    testGetSetOption()
+    {
+        net::io_context ioc;
+        stream<test::stream> ws(ioc);
+
+        {
+            ws.set_option(
+                stream_base::decorator(
+                    [](request_type&)
+                    {
+                    }));
+
+            ws.set_option(
+                stream_base::decorator(
+                    [](response_type&)
+                    {
+                    }));
+        }
+
+        {
+            ws.set_option(
+                stream_base::timeout::suggested(
+                    role_type::client));
+
+            ws.set_option(
+                stream_base::timeout::suggested(
+                    role_type::server));
+
+            ws.set_option({
+                std::chrono::seconds(30),
+                std::chrono::seconds(300),
+                true});
+
+            stream_base::timeout opt;
+            ws.get_option(opt);
+            ws.set_option(opt);
+        }
+    }
+
     void
     testOptions()
     {
@@ -29,12 +72,12 @@ public:
 
         stream<test::stream> ws{ioc_};
         ws.auto_fragment(true);
-        ws.write_buffer_size(2048);
+        ws.write_buffer_bytes(2048);
         ws.binary(false);
         ws.read_message_max(1 * 1024 * 1024);
         try
         {
-            ws.write_buffer_size(7);
+            ws.write_buffer_bytes(7);
             fail();
         }
         catch(std::exception const&)
@@ -109,33 +152,45 @@ public:
         }
     }
 
+    void
+    testJavadoc()
+    {
+        net::io_context ioc;
+        {
+            websocket::stream<tcp_stream> ws{net::make_strand(ioc)};
+        }
+        {
+            websocket::stream<tcp_stream> ws(ioc);
+        }
+    }
+
     void
     run() override
     {
         BOOST_STATIC_ASSERT(std::is_constructible<
-            stream<test::stream>, boost::asio::io_context&>::value);
+            stream<test::stream>, net::io_context&>::value);
 
         BOOST_STATIC_ASSERT(std::is_move_constructible<
             stream<test::stream>>::value);
 
+    #if 0
         BOOST_STATIC_ASSERT(std::is_move_assignable<
             stream<test::stream>>::value);
+    #endif
 
         BOOST_STATIC_ASSERT(std::is_constructible<
             stream<test::stream&>, test::stream&>::value);
 
+        // VFALCO Should these be allowed for NextLayer references?
         BOOST_STATIC_ASSERT(std::is_move_constructible<
             stream<test::stream&>>::value);
-
-        BOOST_STATIC_ASSERT(! std::is_move_assignable<
+    #if 0
+        BOOST_STATIC_ASSERT(std::is_move_assignable<
             stream<test::stream&>>::value);
-
-        log << "sizeof(websocket::stream_base<true>) == " <<
-            sizeof(websocket::detail::stream_base<true>) << std::endl;
-        log << "sizeof(websocket::stream) == " <<
-            sizeof(websocket::stream<test::stream&>) << std::endl;
+    #endif
 
         testOptions();
+        testJavadoc();
     }
 };
 
