@@ -13,6 +13,7 @@
 #endif
 
 #include <boost/container/detail/dlmalloc.hpp>
+#include <boost/container/throw_exception.hpp>
 
 #define BOOST_INTERPROCESS_VECTOR_ALLOC_STATS
 
@@ -20,18 +21,18 @@
 #include <typeinfo>  //typeid
 #include <cassert>   //assert
 
-#include <boost/timer/timer.hpp>
-using boost::timer::cpu_timer;
-using boost::timer::cpu_times;
-using boost::timer::nanosecond_type;
+#include <boost/move/detail/nsec_clock.hpp>
+using boost::move_detail::cpu_timer;
+using boost::move_detail::cpu_times;
+using boost::move_detail::nanosecond_type;
 
 using namespace boost::container;
 
 template <class POD>
-void allocation_timing_test(unsigned int num_iterations, unsigned int num_elements)
+void allocation_timing_test(std::size_t num_iterations, std::size_t num_elements)
 {
    size_t capacity = 0;
-   unsigned int numalloc = 0, numexpand = 0;
+   std::size_t numalloc = 0, numexpand = 0;
 
    std::cout
       << "    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~   \n"
@@ -50,7 +51,7 @@ void allocation_timing_test(unsigned int num_iterations, unsigned int num_elemen
       cpu_timer timer;
       timer.resume();
 
-      for(unsigned int r = 0; r != num_iterations; ++r){
+      for(std::size_t r = 0; r != num_iterations; ++r){
          void *first_mem = 0;
          if(m_mode != BOOST_CONTAINER_EXPAND_FWD)
             first_mem = dlmalloc_malloc(sizeof(POD)*num_elements*3/2);
@@ -61,7 +62,7 @@ void allocation_timing_test(unsigned int num_iterations, unsigned int num_elemen
          dlmalloc_free(first_mem);
          ++numalloc;
 
-         try{
+         BOOST_CONTAINER_TRY{
             dlmalloc_command_ret_t ret;
             for(size_t e = capacity + 1; e < num_elements; ++e){
                size_t received_size;
@@ -73,8 +74,7 @@ void allocation_timing_test(unsigned int num_iterations, unsigned int num_elemen
                   ( m_mode, sizeof(POD)
                   , min, max, &received_size, addr);
                if(!ret.first){
-                  std::cout << "(!ret.first)!" << std::endl;
-                  throw int(0);
+                  throw_runtime_error("!ret.first)");
                }
                if(!ret.second){
                   assert(m_mode == BOOST_CONTAINER_ALLOCATE_NEW);
@@ -100,10 +100,11 @@ void allocation_timing_test(unsigned int num_iterations, unsigned int num_elemen
             }
             dlmalloc_free(addr);
          }
-         catch(...){
+         BOOST_CONTAINER_CATCH(...){
             dlmalloc_free(addr);
-            throw;
+            BOOST_CONTAINER_RETHROW;
          }
+         BOOST_CONTAINER_CATCH_END
       }
 
       assert( dlmalloc_allocated_memory() == 0);
@@ -118,12 +119,12 @@ void allocation_timing_test(unsigned int num_iterations, unsigned int num_elemen
       std::cout   << "  Malloc type:               " << malloc_name
                   << std::endl
                   << "  allocation ns:             "
-                  << float(nseconds)/(num_iterations*num_elements)
+                  << float(nseconds)/float(num_iterations*num_elements)
                   << std::endl
                   << "  capacity  -  alloc calls (new/expand):  "
-                     << (unsigned int)capacity << "  -  "
-                     << (float(numalloc) + float(numexpand))/num_iterations
-                     << "(" << float(numalloc)/num_iterations << "/" << float(numexpand)/num_iterations << ")"
+                     << (std::size_t)capacity << "  -  "
+                     << (float(numalloc) + float(numexpand))/float(num_iterations)
+                     << "(" << float(numalloc)/float(num_iterations) << "/" << float(numexpand)/float(num_iterations) << ")"
                   << std::endl << std::endl;
       dlmalloc_trim(0);
    }
@@ -150,24 +151,24 @@ int allocation_loop()
    #define SIMPLE_IT
    #ifdef SINGLE_TEST
       #ifdef NDEBUG
-      unsigned int numrep [] = { 50000 };
+      std::size_t numrep [] = { 50000 };
       #else
-      unsigned int numrep [] = { 5000 };
+      std::size_t numrep [] = { 5000 };
       #endif
-      unsigned int numele [] = { 100 };
+      std::size_t numele [] = { 100 };
    #elif defined(SIMPLE_IT)
-      unsigned int numrep [] = { 3 };
-      unsigned int numele [] = { 100 };
+      std::size_t numrep [] = { 3 };
+      std::size_t numele [] = { 100 };
    #else
       #ifdef NDEBUG
-      unsigned int numrep [] = { /*10000, */10000, 100000, 1000000 };
+      std::size_t numrep [] = { /*10000, */10000, 100000, 1000000 };
       #else
-      unsigned int numrep [] = { /*10000, */1000, 10000, 100000 };
+      std::size_t numrep [] = { /*10000, */1000, 10000, 100000 };
       #endif
-      unsigned int numele [] = { /*10000,  */1000,    100,      10 };
+      std::size_t numele [] = { /*10000,  */1000,    100,      10 };
    #endif
 
-   for(unsigned int i = 0; i < sizeof(numele)/sizeof(numele[0]); ++i){
+   for(std::size_t i = 0; i < sizeof(numele)/sizeof(numele[0]); ++i){
       allocation_timing_test<POD>(numrep[i], numele[i]);
    }
 
