@@ -1,25 +1,22 @@
 // Boost.Geometry (aka GGL, Generic Geometry Library)
 // Unit Test
 
-// Copyright (c) 2017-2020 Oracle and/or its affiliates.
-
+// Copyright (c) 2017-2021 Oracle and/or its affiliates.
 // Contributed and/or modified by Vissarion Fysikopoulos, on behalf of Oracle
 // Contributed and/or modified by Adam Wulkiewicz, on behalf of Oracle
 
 // Licensed under the Boost Software License version 1.0.
 // http://www.boost.org/users/license.html
 
+#ifdef BOOST_GEOMETRY_TEST_DEBUG
 #include <iostream>
+#endif
 
 #ifndef BOOST_TEST_MODULE
 #define BOOST_TEST_MODULE test_distance_geographic_linear_areal
 #endif
 
-#include <boost/type_traits/is_same.hpp>
-
 #include <boost/test/included/unit_test.hpp>
-#include <boost/geometry/util/condition.hpp>
-#include <boost/geometry/strategies/strategies.hpp>
 
 #include "test_distance_geo_common.hpp"
 #include "test_empty_geometry.hpp"
