@@ -1,6 +1,6 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2011. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
@@ -8,7 +8,6 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 
-#include <boost/interprocess/detail/config_begin.hpp>
 #include <iostream>
 #include <boost/interprocess/mapped_region.hpp>
 #include <boost/interprocess/anonymous_shared_memory.hpp>
@@ -19,7 +18,7 @@ using namespace boost::interprocess;
 
 int main ()
 {
-   try{
+   BOOST_INTERPROCESS_TRY{
       const std::size_t MemSize = 99999*2;
       {
          //Now check anonymous mapping
@@ -44,11 +43,9 @@ int main ()
          }
       }
    }
-   catch(std::exception &exc){
+   BOOST_INTERPROCESS_CATCH(std::exception &exc){
       std::cout << "Unhandled exception: " << exc.what() << std::endl;
       return 1;
-   }
+   } BOOST_INTERPROCESS_CATCH_END
    return 0;
 }
-
-#include <boost/interprocess/detail/config_end.hpp>
