@@ -1,17 +1,16 @@
 //////////////////////////////////////////////////////////////////////////////
 //
-// (C) Copyright Ion Gaztanaga 2004-2009. Distributed under the Boost
+// (C) Copyright Ion Gaztanaga 2004-2012. Distributed under the Boost
 // Software License, Version 1.0. (See accompanying file
 // LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
 // See http://www.boost.org/libs/interprocess for documentation.
 //
 //////////////////////////////////////////////////////////////////////////////
-
-#include <boost/interprocess/detail/config_begin.hpp>
+#define BOOST_CONTAINER_ADAPTIVE_NODE_POOL_CHECK_INVARIANTS
 #include <boost/interprocess/managed_shared_memory.hpp>
-#include <boost/interprocess/containers/list.hpp>
-#include <boost/interprocess/containers/vector.hpp>
+#include <boost/container/list.hpp>
+#include <boost/container/vector.hpp>
 #include <boost/interprocess/allocators/cached_adaptive_pool.hpp>
 #include "print_container.hpp"
 #include "dummy_test_allocator.hpp"
@@ -31,19 +30,27 @@ typedef ipcdetail::cached_adaptive_pool_v1
    <int, managed_shared_memory::segment_manager>
    cached_node_allocator_v1_t;
 
+namespace boost {
+namespace interprocess {
+
 //Explicit instantiations to catch compilation errors
 template class cached_adaptive_pool<int, managed_shared_memory::segment_manager>;
-template class ipcdetail::cached_adaptive_pool_v1<int, managed_shared_memory::segment_manager>;
 template class cached_adaptive_pool<void, managed_shared_memory::segment_manager>;
+
+namespace ipcdetail {
+
+template class ipcdetail::cached_adaptive_pool_v1<int, managed_shared_memory::segment_manager>;
 template class ipcdetail::cached_adaptive_pool_v1<void, managed_shared_memory::segment_manager>;
 
+}}}
+
 //Alias list types
-typedef list<int, cached_node_allocator_t>    MyShmList;
-typedef list<int, cached_node_allocator_v1_t> MyShmListV1;
+typedef boost::container::list<int, cached_node_allocator_t>    MyShmList;
+typedef boost::container::list<int, cached_node_allocator_v1_t> MyShmListV1;
 
 //Alias vector types
-typedef vector<int, cached_node_allocator_t>    MyShmVector;
-typedef vector<int, cached_node_allocator_v1_t> MyShmVectorV1;
+typedef boost::container::vector<int, cached_node_allocator_t>    MyShmVector;
+typedef boost::container::vector<int, cached_node_allocator_v1_t> MyShmVectorV1;
 
 int main ()
 {
@@ -61,5 +68,3 @@ int main ()
 
    return 0;
 }
-
-#include <boost/interprocess/detail/config_end.hpp>
