@@ -26,6 +26,10 @@
 #include <map>
 #include <unordered_map>
 
+#ifndef BOOST_NO_CXX17_HDR_FILESYSTEM
+# include <filesystem>
+#endif // BOOST_NO_CXX17_HDR_FILESYSTEM
+
 //----------------------------------------------------------
 
 namespace value_from_test_ns
@@ -131,7 +135,7 @@ void
 tag_invoke(
     ::boost::json::value_from_tag,
     ::boost::json::value& jv,
-    ::boost::json::error_code& ec,
+    ::boost::system::error_code& ec,
     T8 const& t8)
 {
     if( t8.error )
@@ -159,7 +163,7 @@ tag_invoke(
     if( t9.num == 0 )
         throw std::invalid_argument("");
     if( t9.num < 0 )
-        throw ::boost::json::system_error(
+        throw ::boost::system::system_error(
             make_error_code(::boost::json::error::syntax));
 
     jv = "T9";
@@ -179,8 +183,48 @@ BOOST_DESCRIBE_STRUCT(T10, (), (n, d))
 struct T11 : T10
 {
     std::string s;
+
+    void
+    set_n(int v)
+    {
+        n = v;
+    }
+
+    void
+    set_d(double v)
+    {
+        d = v;
+    }
+
+    void
+    set_b(bool v)
+    {
+        b = v;
+    }
+
+private:
+    bool b;
+
+    BOOST_DESCRIBE_CLASS(T11, (T10), (s), (), (b))
 };
-BOOST_DESCRIBE_STRUCT(T11, (T10), (s))
+
+struct Base1
+{
+    int a = 1;
+};
+BOOST_DESCRIBE_STRUCT(Base1, (), (a))
+
+struct Base2
+{
+    int a = 2;
+};
+BOOST_DESCRIBE_STRUCT(Base2, (), (a))
+
+struct Derived: Base1, Base2
+{
+    int a = 3;
+};
+BOOST_DESCRIBE_STRUCT(Derived, (Base1, Base2), ())
 
 //----------------------------------------------------------
 
@@ -481,11 +525,12 @@ public:
         BOOST_TEST(( jv == value{{"n", 909}, {"d", -1.45}} ));
 
         ::value_from_test_ns::T11 t11;
-        t11.n = 67;
-        t11.d = -.12;
+        t11.set_n(67);
+        t11.set_d(-.12);
+        t11.set_b(true);
         t11.s = "qwerty";
         jv = value_from( t11, ctx... );
-        BOOST_TEST(( jv == value{{"n", 67}, {"d", -.12}, {"s", "qwerty"}} ));
+        BOOST_TEST(( jv == value{{"n", 67}, {"d", -.12}, {"s", "qwerty"}, {"b", true}} ));
 
         ::value_from_test_ns::E1 e1 = ::value_from_test_ns::E1::a;
         BOOST_TEST( value_from( e1, ctx... ) == "a" );
@@ -535,6 +580,20 @@ public:
 #endif // BOOST_NO_CXX17_HDR_VARIANT
     }
 
+    template< class... Context >
+    static
+    void testPath( Context const& ... ctx )
+    {
+        ignore_unused( ctx... );
+#ifndef BOOST_NO_CXX17_HDR_FILESYSTEM
+        std::vector<std::filesystem::path> paths{
+            "from/here", "to/there", "", "c:/" , "..", "../"};
+        value jv = value_from( paths, ctx... );
+        BOOST_TEST(
+            jv == (value{"from/here", "to/there", "", "c:/" , "..", "../"}) );
+#endif // BOOST_NO_CXX17_HDR_FILESYSTEM
+    }
+
     void
     testContext()
     {
@@ -598,6 +657,7 @@ public:
             testDescribed( Context()... );
             testOptional( Context()... );
             testVariant( Context()... );
+            testPath( Context()... );
         }
     };
 
