@@ -1,5 +1,4 @@
-// Copyright 2018-2023 Emil Dotchevski and Reverge Studios, Inc.
-
+// Copyright 2018-2024 Emil Dotchevski and Reverge Studios, Inc.
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -47,7 +46,7 @@ public:
     ~my_info()
     {
         BOOST_TEST(--object_count>=0);
-        if( value!=-1 )
+        if( value != -1 )
             BOOST_TEST(--value_count>=0);
     }
 };
@@ -85,7 +84,7 @@ public:
 
 void run_tests()
 {
-    using leaf::leaf_detail::optional;
+    using leaf::detail::optional;
     {
         optional<my_info> x;
         BOOST_TEST(x.empty());
@@ -300,7 +299,7 @@ void run_tests()
     {
         optional<my_info> x;
         my_info a(42);
-        x.put(10, a);
+        x.load(10, a);
         BOOST_TEST_EQ(object_count, 2);
         BOOST_TEST_EQ(value_count, 2);
         BOOST_TEST(!x.empty());
@@ -316,7 +315,7 @@ void run_tests()
         BOOST_TEST_EQ(object_count, 1);
         BOOST_TEST_EQ(value_count, 1);
         my_info a(42);
-        x.put(10, a);
+        x.load(10, a);
         BOOST_TEST_EQ(object_count, 2);
         BOOST_TEST_EQ(value_count, 2);
         BOOST_TEST(!x.empty());
@@ -329,7 +328,7 @@ void run_tests()
     {
         optional<my_info> x;
         BOOST_TEST(x.empty());
-        x.put(10, my_info(42));
+        x.load(10, my_info(42));
         BOOST_TEST_EQ(object_count, 1);
         BOOST_TEST_EQ(value_count, 1);
         BOOST_TEST(!x.empty());
@@ -344,7 +343,7 @@ void run_tests()
         BOOST_TEST(!x.empty());
         BOOST_TEST_EQ(object_count, 1);
         BOOST_TEST_EQ(value_count, 1);
-        x.put(10, my_info(42));
+        x.load(10, my_info(42));
         BOOST_TEST(!x.empty());
         BOOST_TEST_EQ(object_count, 1);
         BOOST_TEST_EQ(value_count, 1);
