@@ -3,12 +3,17 @@
 //  Boost Software License, Version 1.0.  (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 //
-#define BOOST_MATH_TR1_SOURCE
+#  include <pch.hpp>
+#ifndef BOOST_MATH_TR1_SOURCE
+#  define BOOST_MATH_TR1_SOURCE
+#endif
 #include <boost/math/tr1.hpp>
 #include <boost/math/special_functions/acosh.hpp>
 #include "c_policy.hpp"
 
-extern "C" double BOOST_MATH_TR1_DECL acosh BOOST_PREVENT_MACRO_SUBSTITUTION(double x)
+extern "C" double BOOST_MATH_TR1_DECL boost_acosh BOOST_MATH_PREVENT_MACRO_SUBSTITUTION(double x) BOOST_MATH_C99_THROW_SPEC
 {
-   return c_policies::acosh BOOST_PREVENT_MACRO_SUBSTITUTION(x);
+   return c_policies::acosh BOOST_MATH_PREVENT_MACRO_SUBSTITUTION(x);
 }
+
+
