@@ -16,16 +16,26 @@
 #  pragma warning(disable: 4127) // conditional expression is constant
 #endif
 
+#ifdef SYCL_LANGUAGE_VERSION
+#include "sycl/sycl.hpp"
+#endif
+
+#include <boost/math/tools/config.hpp>
+#include "../include_private/boost/math/tools/test.hpp"
+
+#ifndef BOOST_MATH_NO_REAL_CONCEPT_TESTS
 #include <boost/math/concepts/real_concept.hpp> // for real_concept
 using ::boost::math::concepts::real_concept;
+#endif
 
 #include <boost/math/distributions/chi_squared.hpp> // for chi_squared_distribution
+#include <boost/math/distributions/non_central_chi_squared.hpp> // for chi_squared_distribution
 using boost::math::chi_squared_distribution;
 using boost::math::chi_squared;
 
 #define BOOST_TEST_MAIN
 #include <boost/test/unit_test.hpp> // for test_main
-#include <boost/test/floating_point_comparison.hpp> // for BOOST_CHECK_CLOSE
+#include <boost/test/tools/floating_point_comparison.hpp> // for BOOST_CHECK_CLOSE
 
 #include "test_out_of_range.hpp"
 
@@ -34,6 +44,8 @@ using std::cout;
 using std::endl;
 #include <limits>
 using std::numeric_limits;
+#include <cmath>
+using std::log;
 
 template <class RealType>
 RealType naive_pdf(RealType df, RealType x)
@@ -57,11 +69,13 @@ void test_spot(
       cdf(dist, cs), P, tol);
    BOOST_CHECK_CLOSE(
       pdf(dist, cs), naive_pdf(dist.degrees_of_freedom(), cs), tol);
+   BOOST_CHECK_CLOSE(
+      logpdf(dist, cs), log(pdf(dist, cs)), tol);
    if((P < 0.99) && (Q < 0.99))
    {
       //
       // We can only check this if P is not too close to 1,
-      // so that we can guarentee Q is free of error:
+      // so that we can guarantee Q is free of error:
       //
       BOOST_CHECK_CLOSE(
          cdf(complement(dist, cs)), Q, tol);
@@ -70,6 +84,25 @@ void test_spot(
       BOOST_CHECK_CLOSE(
             quantile(complement(dist, Q)), cs, tol);
    }
+
+   boost::math::non_central_chi_squared_distribution<RealType> dist2(df, 0);
+   BOOST_CHECK_CLOSE(
+      cdf(dist2, cs), P, tol);
+   BOOST_CHECK_CLOSE(
+      pdf(dist2, cs), naive_pdf(dist2.degrees_of_freedom(), cs), tol);
+   if((P < 0.99) && (Q < 0.99))
+   {
+      //
+      // We can only check this if P is not too close to 1,
+      // so that we can guarantee Q is free of error:
+      //
+      BOOST_CHECK_CLOSE(
+         cdf(complement(dist2, cs)), Q, tol);
+      BOOST_CHECK_CLOSE(
+         quantile(dist2, P), cs, tol);
+      BOOST_CHECK_CLOSE(
+         quantile(complement(dist2, Q)), cs, tol);
+   }
 }
 
 //
@@ -293,7 +326,7 @@ template <class RealType> // Any floating-point type RealType.
 void test_spots(RealType T)
 {
   // Basic sanity checks, test data is to three decimal places only
-  // so set tolerance to 0.001 expressed as a persentage.
+  // so set tolerance to 0.001 expressed as a percentage.
 
   RealType tolerance = 0.001f * 100;
 
@@ -423,7 +456,7 @@ void test_spots(RealType T)
        kurtosis_excess(dist)
        , static_cast<RealType>(1.5), tol2);
     // special cases:
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        pdf(
           chi_squared_distribution<RealType>(static_cast<RealType>(1)),
           static_cast<RealType>(0)), std::overflow_error
@@ -453,62 +486,62 @@ void test_spots(RealType T)
        cdf(complement(chi_squared_distribution<RealType>(3), static_cast<RealType>(0)))
        , static_cast<RealType>(1));
 
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        pdf(
           chi_squared_distribution<RealType>(static_cast<RealType>(-1)),
           static_cast<RealType>(1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        pdf(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(-1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        cdf(
           chi_squared_distribution<RealType>(static_cast<RealType>(-1)),
           static_cast<RealType>(1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        cdf(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(-1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        cdf(complement(
           chi_squared_distribution<RealType>(static_cast<RealType>(-1)),
           static_cast<RealType>(1))), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        cdf(complement(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(-1))), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(
           chi_squared_distribution<RealType>(static_cast<RealType>(-1)),
           static_cast<RealType>(0.5)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(-1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(1.1)), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(complement(
           chi_squared_distribution<RealType>(static_cast<RealType>(-1)),
           static_cast<RealType>(0.5))), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(complement(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(-1))), std::domain_error
        );
-    BOOST_CHECK_THROW(
+    BOOST_MATH_CHECK_THROW(
        quantile(complement(
           chi_squared_distribution<RealType>(static_cast<RealType>(8)),
           static_cast<RealType>(1.1))), std::domain_error
