@@ -23,19 +23,28 @@
 #  pragma warning(disable: 4127) // conditional expression is constant.
 #endif
 
+#ifdef SYCL_LANGUAGE_VERSION
+#include "sycl/sycl.hpp"
+#endif
+
+#include <boost/math/tools/config.hpp>
+
 #define BOOST_TEST_MAIN
 #include <boost/test/unit_test.hpp> // Boost.Test
-#include <boost/test/floating_point_comparison.hpp>
+#include <boost/test/tools/floating_point_comparison.hpp>
 
+#ifndef BOOST_MATH_NO_REAL_CONCEPT_TESTS
 #include <boost/math/concepts/real_concept.hpp> // for real_concept
+#endif
+
 #include <boost/math/distributions/poisson.hpp>
     using boost::math::poisson_distribution;
-#include <boost/math/tools/test.hpp> // for real_concept
 
 #include <boost/math/special_functions/gamma.hpp> // for (incomplete) gamma.
 //   using boost::math::qamma_Q;
 #include "table_type.hpp"
 #include "test_out_of_range.hpp"
+#include "../include_private/boost/math/tools/test.hpp"
 
 #include <iostream>
    using std::cout;
@@ -53,12 +62,12 @@ void test_spots(RealType)
    // guaranteed for type RealType, eg 6 for float, 15 for double,
    // expressed as a percentage (so -2) for BOOST_CHECK_CLOSE,
 
-   int decdigits = numeric_limits<RealType>::digits10;
-  // May eb >15 for 80 and 128-bit FP typtes.
+   int decdigits = std::numeric_limits<RealType>::digits10;
+  // May eb >15 for 80 and 128-bit FP types.
   if (decdigits <= 0)
   { // decdigits is not defined, for example real concept,
     // so assume precision of most test data is double (for example, MathCAD).
-     decdigits = numeric_limits<double>::digits10; // == 15 for 64-bit
+     decdigits = std::numeric_limits<double>::digits10; // == 15 for 64-bit
   }
   if (decdigits > 15 ) // numeric_limits<double>::digits10)
   { // 15 is the accuracy of the MathCAD test data.
@@ -90,7 +99,7 @@ void test_spots(RealType)
   // ppois(10, 1) = 9.999999899522340E-001
   // ppois(5,5) = 0.615960654833065
 
-  // qpois returns inverse Poission distribution, that is the smallest (floor) k so that ppois(k, lambda) >= p
+  // qpois returns inverse Poisson distribution, that is the smallest (floor) k so that ppois(k, lambda) >= p
   // p is real number, real mean lambda > 0
   // k is approximately the integer for which probability(X <= k) = p
   // when random variable X has the Poisson distribution with parameters lambda.
@@ -106,6 +115,7 @@ void test_spots(RealType)
   using  ::boost::math::pdf;
 
    // Check that bad arguments throw.
+   #ifndef BOOST_MATH_NO_EXCEPTIONS
    BOOST_MATH_CHECK_THROW(
    cdf(poisson_distribution<RealType>(static_cast<RealType>(0)), // mean zero is bad.
       static_cast<RealType>(0)),  // even for a good k.
@@ -155,6 +165,7 @@ void test_spots(RealType)
      quantile(complement(poisson_distribution<RealType>(static_cast<RealType>(1)), 
       static_cast<RealType>(0))),  // bad probability. 
       std::overflow_error);
+   #endif
 
   BOOST_CHECK_EQUAL(
      quantile(poisson_distribution<RealType>(static_cast<RealType>(1)), 
@@ -208,6 +219,31 @@ void test_spots(RealType)
       static_cast<RealType>(20)),   //  K>> mean 
       static_cast<RealType>(8.277463646553730E-009), // probability.
          tolerance);
+
+  // LOGPDF
+  BOOST_CHECK_CLOSE(
+     logpdf(poisson_distribution<RealType>(static_cast<RealType>(4)), // mean 4.
+      static_cast<RealType>(0)),   
+      log(static_cast<RealType>(1.831563888873410E-002)), // probability.
+         tolerance);
+
+  BOOST_CHECK_CLOSE(
+     logpdf(poisson_distribution<RealType>(static_cast<RealType>(4)), // mean 4.
+      static_cast<RealType>(2)),   
+      log(static_cast<RealType>(1.465251111098740E-001)), // probability.
+         tolerance);
+
+  BOOST_CHECK_CLOSE(
+     logpdf(poisson_distribution<RealType>(static_cast<RealType>(20)), // mean big.
+      static_cast<RealType>(1)),   //  k small
+      log(static_cast<RealType>(4.122307244877130E-008)), // probability.
+         tolerance);
+
+  BOOST_CHECK_CLOSE(
+     logpdf(poisson_distribution<RealType>(static_cast<RealType>(4)), // mean 4.
+      static_cast<RealType>(20)),   //  K>> mean 
+      log(static_cast<RealType>(8.277463646553730E-009)), // probability.
+         tolerance);
   
   // CDF
   BOOST_CHECK_CLOSE(
@@ -449,7 +485,7 @@ void test_spots(RealType)
   for(unsigned i = 0; i < poisson_quantile_data.size(); ++i)
   {
      using namespace boost::math::policies;
-     typedef policy<discrete_quantile<real> > P1;
+     typedef policy<discrete_quantile<boost::math::policies::real> > P1;
      typedef policy<discrete_quantile<integer_round_down> > P2;
      typedef policy<discrete_quantile<integer_round_up> > P3;
      typedef policy<discrete_quantile<integer_round_outwards> > P4;
@@ -534,6 +570,7 @@ BOOST_AUTO_TEST_CASE( test_main )
   // poisson mydudpoisson(0.);
   // throws (if BOOST_MATH_DOMAIN_ERROR_POLICY == throw_on_error).
 
+#ifndef BOOST_MATH_NO_EXCEPTIONS
 #ifndef BOOST_NO_EXCEPTIONS
   BOOST_MATH_CHECK_THROW(poisson mydudpoisson(-1), std::domain_error);// Mean must be > 0.
   BOOST_MATH_CHECK_THROW(poisson mydudpoisson(-1), std::logic_error);// Mean must be > 0.
@@ -545,7 +582,7 @@ BOOST_AUTO_TEST_CASE( test_main )
   // BOOST_MATH_CHECK_THROW(poisson mydudpoisson(-1), std::overflow_error); // fails the check
   // because overflow_error is unrelated - except from std::exception
   BOOST_MATH_CHECK_THROW(cdf(mypoisson, -1), std::domain_error); // k must be >= 0
-
+#endif
   BOOST_CHECK_EQUAL(mean(mypoisson), 4.);
   BOOST_CHECK_CLOSE(
   pdf(mypoisson, 2.),  // k events = 2. 
@@ -619,7 +656,7 @@ BOOST_AUTO_TEST_CASE( test_main )
   test_spots(0.0); // Test double.
 #endif
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
-  if (numeric_limits<long double>::digits10 > numeric_limits<double>::digits10)
+  if (std::numeric_limits<long double>::digits10 > std::numeric_limits<double>::digits10)
   { // long double is better than double (so not MSVC where they are same).
 #ifdef TEST_LDOUBLE
      test_spots(0.0L); // Test long double.
