@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 #
-# Copyright (c) 2019-2024 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+# Copyright (c) 2019-2025 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 #
 # Distributed under the Boost Software License, Version 1.0. (See accompanying
 # file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -12,6 +12,12 @@ from os import path
 from typing import List, Tuple
 import glob
 from abc import abstractmethod, ABCMeta
+import sys
+
+THIS_FOLDER = path.abspath(path.dirname(path.realpath(__file__)))
+sys.path.append(path.join(THIS_FOLDER))
+
+import examples_qbk
 
 # Script to get file headers (copyright notices
 # and include guards) okay and up to date
@@ -32,12 +38,13 @@ BASE_FOLDERS = [
 ]
 BASE_FILES = [
     'CMakeLists.txt',
-    '.drone.star'
+    '.drone.star',
+    '.codecov.yml'
 ]
 HTML_GEN_PATH = path.join(REPO_BASE, 'doc', 'html')
 
 HEADER_TEMPLATE = '''{begin}
-{linesym} Copyright (c) 2019-2024 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
+{linesym} Copyright (c) 2019-2025 Ruben Perez Hidalgo (rubenperez038 at gmail dot com)
 {linesym}
 {linesym} Distributed under the Boost Software License, Version 1.0. (See accompanying
 {linesym} file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -61,8 +68,8 @@ SRC_HPP_TEMPLATE = '''
 MYSQL_ERROR_HEADER = path.join(REPO_BASE, 'private', 'mysqld_error.h')
 MARIADB_ERROR_HEADER = path.join(REPO_BASE, 'private', 'mariadb_error.h')
 
-def find_first_blank(lines):
-    return [i for i, line in enumerate(lines) if line == ''][0]
+def find_first_blank(lines: List[str]) -> int:
+    return next((i for i, line in enumerate(lines) if line.replace('\n', '') == ''), len(lines))
 
 def read_file(fpath):
     with open(fpath, 'rt') as f:
@@ -106,7 +113,7 @@ class NormalProcessor(BaseProcessor):
         self.name = name
         
     def process(self, lines: List[str], _: str) -> List[str]:
-        first_blank = find_first_blank(line.replace('\n', '') for line in lines)
+        first_blank = find_first_blank(lines)
         lines = self.header + lines[first_blank:]
         return lines
         
@@ -153,10 +160,11 @@ class MysqlHppProcessor(HppProcessor):
 
     def process(self, _: List[str], fpath: str) -> List[str]:
         base = Path(REPO_BASE)
+        exclusions = ('src.hpp', 'pfr.hpp')
         includes = [
             fname.relative_to(base.joinpath('include'))
             for fname in sorted(base.joinpath('include', 'boost', 'mysql').glob('*.hpp'))
-            if fname.name != 'src.hpp'
+            if fname.name not in exclusions
         ]
         return gen_header('//', include_guard=self._gen_include_guard(fpath)) + \
             ['\n'] + \
@@ -227,21 +235,28 @@ FILE_PROCESSORS : List[Tuple[str, BaseProcessor]] = [
     ('.tar.gz', IgnoreProcessor()),
     ('.json', IgnoreProcessor()),
     ('.txt', IgnoreProcessor()),
+    ('.pyc', IgnoreProcessor()),
+    ('.ipynb', IgnoreProcessor()),
+    ('.png', IgnoreProcessor()),
 ]
 
 def process_file(fpath: str):
-    for ext, processor in FILE_PROCESSORS:
-        if fpath.endswith(ext):
-            if VERBOSE:
-                print('Processing file {} with processor {}'.format(fpath, processor.name))
-            if not processor.skip:
-                lines = read_file(fpath)
-                output_lines = processor.process(lines, fpath)
-                if output_lines != lines:
-                    write_file(fpath, output_lines)
-            break
-    else:
-        raise ValueError('Could not find a suitable processor for file: ' + fpath)
+    try:
+        for ext, processor in FILE_PROCESSORS:
+            if fpath.endswith(ext):
+                if VERBOSE:
+                    print('Processing file {} with processor {}'.format(fpath, processor.name))
+                if not processor.skip:
+                    lines = read_file(fpath)
+                    output_lines = processor.process(lines, fpath)
+                    if output_lines != lines:
+                        write_file(fpath, output_lines)
+                break
+        else:
+            raise ValueError('Could not find a suitable processor for file: ' + fpath)
+    except Exception:
+        print(f'Found error processing {fpath}')
+        raise
     
 def process_all_files():
     for base_folder in BASE_FOLDERS:
@@ -262,7 +277,7 @@ def verify_test_consistency():
     for test_type in ('unit', 'integration'):
         for ftocheck in ('Jamfile', 'CMakeLists.txt'):
             base_path = path.join(REPO_BASE, 'test', test_type)
-            tests = glob.glob(base_path + '/**/*.cpp', recursive=True)
+            tests = glob.glob(base_path + '/test/**/*.cpp', recursive=True)
             tests = [elm.replace(base_path + '/', '') for elm in tests]
 
             with open(path.join(REPO_BASE, 'test', test_type, ftocheck), 'rt') as f:
@@ -277,6 +292,7 @@ def verify_test_consistency():
 def main():
     process_all_files()
     verify_test_consistency()
+    examples_qbk.main()
             
             
 if __name__ == '__main__':
