@@ -1,7 +1,6 @@
-//Copyright (c) 2008-2016 Emil Dotchevski and Reverge Studios, Inc.
-
-//Distributed under the Boost Software License, Version 1.0. (See accompanying
-//file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+// Copyright 2008-2024 Emil Dotchevski and Reverge Studios, Inc.
+// Distributed under the Boost Software License, Version 1.0. (See accompanying
+// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #ifdef BOOST_QVM_TEST_SINGLE_HEADER
 #   include BOOST_QVM_TEST_SINGLE_HEADER
@@ -35,7 +34,7 @@ namespace
             test_qvm::quaternion<Q1> q3(42,1);
             test_qvm::quaternion<Q1> r1=q3*q1;
             rotate_z(q3,r);
-            BOOST_QVM_TEST_EQ(q3.a,r1.a);
+            BOOST_QVM_TEST_CLOSE(q3.a,r1.a,0.000001f);
             }
         rotz_quat(0.0f)+rotz_quat(0.0f);
         -rotz_quat(0.0f);
