@@ -5,12 +5,17 @@
  */
 
 #include <boost/redis/connection.hpp>
-#include <boost/system/errc.hpp>
-#define BOOST_TEST_MODULE conn-quit
+
+#include <boost/system/error_code.hpp>
+
+#include <cstddef>
+#define BOOST_TEST_MODULE conn_quit
 #include <boost/test/included/unit_test.hpp>
-#include <iostream>
+
 #include "common.hpp"
 
+#include <iostream>
+
 namespace net = boost::asio;
 using boost::redis::connection;
 using boost::system::error_code;
@@ -20,24 +25,6 @@ using boost::redis::response;
 using boost::redis::ignore;
 using namespace std::chrono_literals;
 
-BOOST_AUTO_TEST_CASE(test_eof_no_error)
-{
-   request req;
-   req.get_config().cancel_on_connection_lost = false;
-   req.push("QUIT");
-
-   net::io_context ioc;
-   auto conn = std::make_shared<connection>(ioc);
-
-   conn->async_exec(req, ignore, [&](auto ec, auto) {
-      BOOST_TEST(!ec);
-      conn->cancel(operation::reconnection);
-   });
-
-   run(conn);
-   ioc.run();
-}
-
 // Test if quit causes async_run to exit.
 BOOST_AUTO_TEST_CASE(test_async_run_exits)
 {
@@ -58,23 +45,25 @@ BOOST_AUTO_TEST_CASE(test_async_run_exits)
    req3.get_config().cancel_if_not_connected = true;
    req3.push("PING");
 
-   auto c3 = [](auto ec, auto)
-   {
+   bool c1_called = false, c2_called = false, c3_called = false;
+
+   auto c3 = [&](error_code ec, std::size_t) {
+      c3_called = true;
       std::clog << "c3: " << ec.message() << std::endl;
-      BOOST_CHECK_EQUAL(ec, boost::asio::error::operation_aborted);
+      BOOST_TEST(ec == net::error::operation_aborted);
    };
 
-   auto c2 = [&](auto ec, auto)
-   {
+   auto c2 = [&](error_code ec, std::size_t) {
+      c2_called = true;
       std::clog << "c2: " << ec.message() << std::endl;
-      BOOST_TEST(!ec);
+      BOOST_TEST(ec == error_code());
       conn->async_exec(req3, ignore, c3);
    };
 
-   auto c1 = [&](auto ec, auto)
-   {
+   auto c1 = [&](error_code ec, std::size_t) {
+      c1_called = true;
       std::cout << "c1: " << ec.message() << std::endl;
-      BOOST_TEST(!ec);
+      BOOST_TEST(ec == error_code());
       conn->async_exec(req2, ignore, c2);
    };
 
@@ -87,6 +76,9 @@ BOOST_AUTO_TEST_CASE(test_async_run_exits)
    cfg.reconnect_wait_interval = 0s;
    run(conn, cfg);
 
-   ioc.run();
-}
+   ioc.run_for(test_timeout);
 
+   BOOST_TEST(c1_called);
+   BOOST_TEST(c2_called);
+   BOOST_TEST(c3_called);
+}
