@@ -5,17 +5,17 @@
 # License, Version 1.0. (See accompanying file LICENSE_1_0.txt or copy at
 # http://www.boost.org/LICENSE_1_0.txt)
 
-# runtest.sh --toolset=gcc --preserve-test-targets variant=profile
+# profile.sh --toolset=gcc preserve-test-targets=on variant=profile
 if test $# -eq 0
 then
     echo "Usage: $0 <bjam arguments>"
-    echo "Typical bjam arguements are:"
-    echo "  --toolset=msvc-7.1,gcc"
+    echo "Typical bjam arguments are:"
+    echo "  toolset=msvc-7.1,gcc"
     echo "  link=static,shared"
     echo "  threading=single,multi"
     echo "  -sBOOST_ARCHIVE_LIST=<archive name>"
 else
-    bjam --dump-tests --preserve-test-targets variant=profile $@ >bjam.log 2>&1
+    bjam --dump-tests variant=profile preserve-test-targets=on $@ >bjam.log 2>&1
     process_jam_log --v2 <bjam.log
 
     # for each test directory
