@@ -1,4 +1,4 @@
-//  Copyright (c) 2001-2009 Hartmut Kaiser
+//  Copyright (c) 2001-2010 Hartmut Kaiser
 // 
 //  Distributed under the Boost Software License, Version 1.0. (See accompanying 
 //  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -15,8 +15,6 @@
 //  use a larger value for the alignment field width (default is 10)
 #define BOOST_KARMA_DEFAULT_FIELD_LENGTH 25
 
-#include <boost/config/warning_disable.hpp>
-
 #include <iostream>
 #include <string>
 #include <vector>
@@ -25,9 +23,10 @@
 #include <algorithm>
 #include <cstdlib> 
 
-#include <boost/range.hpp>
+#include <boost/array.hpp>
 #include <boost/date_time/gregorian/gregorian.hpp>
 #include <boost/fusion/include/std_pair.hpp>
+#include <boost/range/iterator_range.hpp>
 
 ///////////////////////////////////////////////////////////////////////////////
 // This streaming operator is needed to generate the output from the map below
@@ -43,7 +42,7 @@ namespace std {
 }
 
 #include <boost/spirit/include/karma.hpp>
-#include <boost/spirit/include/karma_stream.hpp>
+#include <boost/spirit/include/karma_format.hpp>
 
 using namespace boost::spirit;
 using namespace boost::spirit::ascii;
@@ -57,19 +56,26 @@ using namespace boost::spirit::ascii;
 template <typename Container>
 void output_container(std::ostream& os, Container const& c)
 {
+    // output the container as a space separated sequence
+    os << 
+        karma::format(
+            *stream,                              // format description
+            c                                     // data
+        ) << std::endl << std::endl;
+
     // output the container as a space separated sequence
     os << 
         karma::format_delimited(
             *stream,                              // format description
-            c,                                    // data
-            space                                 // delimiter
+            space,                                // delimiter
+            c                                     // data
         ) << std::endl << std::endl;
 
     os << 
         karma::format_delimited(
             '[' << *stream << ']',                // format description
-            c,                                    // data
-            space                                 // delimiter
+            space,                                // delimiter
+            c                                     // data
         ) << std::endl << std::endl;
 
     // output the container as a comma separated list
@@ -91,6 +97,12 @@ void output_container(std::ostream& os, Container const& c)
             c                                     // data
         ) << std::endl << std::endl;
 
+    os << 
+        karma::format(
+            '[' << (+stream | "empty") << ']',    // format description
+            c                                     // data
+        ) << std::endl << std::endl;
+
     // output the container as a comma separated list of items enclosed in '()'
     os << 
         karma::format(
@@ -105,15 +117,15 @@ void output_container(std::ostream& os, Container const& c)
              )  << ']',                           // format description
             c                                     // data
         ) << std::endl << std::endl;
-        
+
     // output the container as a HTML list
     os << 
         karma::format_delimited(
             "<ol>" << 
                 *verbatim["<li>" << stream << "</li>"]
             << "</ol>",                           // format description
-            c,                                    // data
-            '\n'                                  // delimiter
+            '\n',                                 // delimiter
+            c                                     // data
         ) << std::endl;
 
     // output the container as right aligned column
@@ -122,8 +134,8 @@ void output_container(std::ostream& os, Container const& c)
            *verbatim[
                 "|" << right_align[stream] << "|"
             ],                                    // format description
-            c,                                    // data
-            '\n'                                  // delimiter
+            '\n',                                 // delimiter
+            c                                     // data
         ) << std::endl;
 
     os << std::endl;
@@ -131,16 +143,25 @@ void output_container(std::ostream& os, Container const& c)
 
 int main()
 {
+    ///////////////////////////////////////////////////////////////////////////
+    // C-style array
+    int i[4] = { 3, 6, 9, 12 };
+    
+    std::cout << "-------------------------------------------------------------" 
+              << std::endl;
+    std::cout << "int i[]" << std::endl;
+    output_container(std::cout, boost::make_iterator_range(i, i+4));
+
     ///////////////////////////////////////////////////////////////////////////
     // vector
-    std::vector<int> v (8);
+    std::vector<int> v (5);
     std::generate(v.begin(), v.end(), std::rand); // randomly fill the vector
 
     std::cout << "-------------------------------------------------------------" 
               << std::endl;
     std::cout << "std::vector<int>" << std::endl;
     output_container(std::cout, v);
-        
+
     ///////////////////////////////////////////////////////////////////////////
     // list
     std::list<char> l;
@@ -153,15 +174,6 @@ int main()
     std::cout << "std::list<char>" << std::endl;
     output_container(std::cout, l);
 
-    ///////////////////////////////////////////////////////////////////////////
-    // C-style array
-    int i[4] = { 3, 6, 9, 12 };
-    
-    std::cout << "-------------------------------------------------------------" 
-              << std::endl;
-    std::cout << "int i[]" << std::endl;
-    output_container(std::cout, boost::make_iterator_range(i, i+4));
-    
     ///////////////////////////////////////////////////////////////////////////
     // strings
     std::string str("Hello world!");
@@ -170,7 +182,17 @@ int main()
               << std::endl;
     std::cout << "std::string" << std::endl;
     output_container(std::cout, str);
-        
+
+    ///////////////////////////////////////////////////////////////////////////
+    // boost::array
+    boost::array<long, 5> arr;
+    std::generate(arr.begin(), arr.end(), std::rand); // randomly fill the array
+
+    std::cout << "-------------------------------------------------------------" 
+              << std::endl;
+    std::cout << "boost::array<long, 5>" << std::endl;
+    output_container(std::cout, arr);
+
     ///////////////////////////////////////////////////////////////////////////
     //  vector of boost::date objects
     //  Note: any registered facets get used!
@@ -187,7 +209,7 @@ int main()
               << std::endl;
     std::cout << "std::vector<boost::date>" << std::endl;
     output_container(std::cout, dates);
-    
+
     ///////////////////////////////////////////////////////////////////////////
     //  map of int --> string mappings
     std::map<int, std::string> mappings;
