@@ -5,13 +5,13 @@
     
     http://www.boost.org/
 
-    Copyright (c) 2001-2011 Hartmut Kaiser. Distributed under the Boost 
+    Copyright (c) 2001-2012 Hartmut Kaiser. Distributed under the Boost 
     Software License, Version 1.0. (See accompanying file 
     LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
 
-#if !defined(SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED)
-#define SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED
+#if !defined(BOOST_SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED)
+#define BOOST_SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED
 
 #include <iomanip>
 #include <ios>
@@ -20,6 +20,7 @@
 #include <boost/wave/token_ids.hpp>  
 #include <boost/wave/language_support.hpp>
 #include <boost/wave/util/file_position.hpp>
+#include <boost/optional.hpp>
 
 ///////////////////////////////////////////////////////////////////////////////
 namespace boost {
@@ -61,12 +62,19 @@ public:
     operator token_id() const { return id; }
     string_type const &get_value() const { return value; }
     position_type const &get_position() const { return pos; }
+    position_type const &get_expand_position() const {
+        if (expand_pos)
+            return *expand_pos;
+        else
+            return pos;
+    }
     bool is_eoi() const { return id == T_EOI; }
     bool is_valid() const { return id != T_UNKNOWN; }
 
     void set_token_id (token_id id_) { id = id_; }
     void set_value (string_type const &newval) { value = newval; }
     void set_position (position_type const &pos_) { pos = pos_; }
+    void set_expand_position (position_type const &pos_) { expand_pos = pos_; }
 
     friend bool operator== (slex_token const& lhs, slex_token const& rhs)
     {
@@ -92,7 +100,7 @@ public:
         
         stream << std::setw(16) 
             << std::left << boost::wave::get_token_name(id) << " ("
-            << "#" << token_id(ID_FROM_TOKEN(*this)) 
+            << "#" << token_id(BASEID_FROM_TOKEN(*this)) 
             << ") at " << get_position().get_file() << " (" 
             << std::setw(3) << std::right << get_position().get_line() << "/" 
             << std::setw(2) << std::right << get_position().get_column() 
@@ -115,6 +123,7 @@ private:
     boost::wave::token_id id;   // the token id
     string_type value;             // the text, which was parsed into this token
     PositionT pos;              // the original file position
+    boost::optional<PositionT> expand_pos;
 };
 
 template <typename PositionT>
@@ -142,4 +151,4 @@ token_is_valid(slex_token<Position> const& t)
 }   // namespace wave
 }   // namespace boost
 
-#endif // !defined(SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED)
+#endif // !defined(BOOST_SLEX_TOKEN_HPP_53A13BD2_FBAA_444B_9B8B_FCB225C2BBA8_INCLUDED)
